@@ -28,7 +28,6 @@
 #include "../rsvg-private.h"
 #include "../rsvg-styles.h"
 #include "../rsvg-css.h"
-#include "../rsvg-drawing-ctx.h"
 #include "common.h"
 
 typedef struct _RsvgFilterPrimitiveGaussianBlur
@@ -582,7 +581,11 @@ gaussian_blur_surface (cairo_surface_t *in,
 }
 
 static void
-rsvg_filter_primitive_gaussian_blur_render (RsvgNode *node, RsvgComputedValues *values, RsvgFilterPrimitive *primitive, RsvgFilterContext *ctx)
+rsvg_filter_primitive_gaussian_blur_render (RsvgNode *node,
+                                            RsvgComputedValues *values,
+                                            RsvgFilterPrimitive *primitive,
+                                            RsvgFilterContext *ctx,
+                                            RsvgDrawingCtx *draw_ctx)
 {
     RsvgFilterPrimitiveGaussianBlur *gaussian = (RsvgFilterPrimitiveGaussianBlur *) primitive;
 
@@ -593,9 +596,9 @@ rsvg_filter_primitive_gaussian_blur_render (RsvgNode *node, RsvgComputedValues *
     RsvgFilterPrimitiveOutput op;
     cairo_t *cr;
 
-    boundarys = rsvg_filter_primitive_get_bounds (primitive, ctx);
+    boundarys = rsvg_filter_primitive_get_bounds (primitive, ctx, draw_ctx);
 
-    in = rsvg_filter_get_in (primitive->in, ctx);
+    in = rsvg_filter_get_in (primitive->in, ctx, draw_ctx);
     if (in == NULL) {
         return;
     }
@@ -638,7 +641,6 @@ rsvg_filter_primitive_gaussian_blur_render (RsvgNode *node, RsvgComputedValues *
     rsvg_filter_store_output (primitive->result, op, ctx);
 
     cairo_surface_destroy (in);
-    cairo_surface_destroy (output);
 }
 
 static void
