@@ -34,33 +34,6 @@ typedef struct  {
     int x0, y0, x1, y1;
 } RsvgIRect;
 
-// typedef RsvgCoordUnits RsvgFilterUnits;
-// 
-// struct _RsvgFilter {
-//     RsvgLength x, y, width, height;
-//     RsvgFilterUnits filterunits;
-//     RsvgFilterUnits primitiveunits;
-// };
-
-/**
- * rsvg_filter_render:
- * @node: a pointer to the filter node to use
- * @source: the a #cairo_surface_t of type %CAIRO_SURFACE_TYPE_IMAGE
- * @context: the context
- *
- * Create a new surface applied the filter. This function will create
- * a context for itself, set up the coordinate systems execute all its
- * little primatives and then clean up its own mess.
- *
- * Returns: (transfer full): a new #cairo_surface_t
- **/
-/* Implemented in rust/src/filters/ffi.rs */
-G_GNUC_INTERNAL
-cairo_surface_t *rsvg_filter_render (RsvgNode *filter_node,
-                                     cairo_surface_t *source,
-                                     RsvgDrawingCtx *context,
-                                     char *channelmap);
-
 G_GNUC_INTERNAL
 RsvgNode    *rsvg_new_filter_primitive_blend                (const char *element_name, RsvgNode *parent, const char *id, const char *klass);
 G_GNUC_INTERNAL
@@ -80,14 +53,10 @@ RsvgNode    *rsvg_new_node_component_transfer_function      (const char *element
 G_GNUC_INTERNAL
 RsvgNode    *rsvg_new_filter_primitive_erode                (const char *element_name, RsvgNode *parent, const char *id, const char *klass);
 G_GNUC_INTERNAL
-RsvgNode    *rsvg_new_filter_primitive_flood                (const char *element_name, RsvgNode *parent, const char *id, const char *klass);
-G_GNUC_INTERNAL
 RsvgNode    *rsvg_new_filter_primitive_displacement_map     (const char *element_name, RsvgNode *parent, const char *id, const char *klass);
 G_GNUC_INTERNAL
 RsvgNode    *rsvg_new_filter_primitive_turbulence           (const char *element_name, RsvgNode *parent, const char *id, const char *klass);
 G_GNUC_INTERNAL
-RsvgNode    *rsvg_new_filter_primitive_image                (const char *element_name, RsvgNode *parent, const char *id, const char *klass);
-G_GNUC_INTERNAL
 RsvgNode    *rsvg_new_filter_primitive_diffuse_lighting	    (const char *element_name, RsvgNode *parent, const char *id, const char *klass);
 G_GNUC_INTERNAL
 RsvgNode    *rsvg_new_node_light_source	                    (const char *element_name, RsvgNode *parent, const char *id, const char *klass);
