@@ -122,7 +122,6 @@
 
 #include "rsvg-private.h"
 #include "rsvg-defs.h"
-#include "rsvg-drawing-ctx.h"
 #include "rsvg-structure.h"
 
 enum {
@@ -771,6 +770,20 @@ rsvg_handle_cascade (RsvgHandle *handle)
     }
 }
 
+static RsvgDrawingCtx *
+rsvg_handle_create_drawing_ctx(RsvgHandle *handle,
+                               cairo_t *cr,
+                               RsvgDimensionData *dimensions)
+{
+    return rsvg_drawing_ctx_new (cr,
+                                 dimensions->width, dimensions->height,
+                                 dimensions->em, dimensions->ex,
+                                 handle->priv->dpi_x, handle->priv->dpi_y,
+                                 handle->priv->defs,
+                                 handle->priv->is_testing);
+
+}
+
 /**
  * rsvg_handle_render_cairo_sub:
  * @handle: A #RsvgHandle
@@ -820,14 +833,11 @@ rsvg_handle_render_cairo_sub (RsvgHandle * handle, cairo_t * cr, const char *id)
     if (dimensions.width == 0 || dimensions.height == 0)
         return FALSE;
 
-    draw = rsvg_drawing_ctx_new (cr,
-                                 dimensions.width, dimensions.height,
-                                 dimensions.em, dimensions.ex,
-                                 handle->priv->dpi_x, handle->priv->dpi_y,
-                                 handle->priv->defs,
-                                 handle->priv->is_testing);
+    draw = rsvg_handle_create_drawing_ctx (handle, cr, &dimensions);
 
-    rsvg_drawing_ctx_add_node_and_ancestors_to_stack (draw, drawsub);
+    if (drawsub != NULL) {
+        rsvg_drawing_ctx_add_node_and_ancestors_to_stack (draw, drawsub);
+    }
 
     cairo_save (cr);
 
@@ -941,23 +951,16 @@ rsvg_handle_get_dimensions_sub (RsvgHandle * handle, RsvgDimensionData * dimensi
 
     if (handle_subelement == TRUE) {
         RsvgDimensionData dimensions;
-        RsvgBbox *bbox;
         cairo_rectangle_t ink_rect;
 
         rsvg_handle_get_dimensions (handle, &dimensions);
         if (dimensions.width == 0 || dimensions.height == 0)
             return FALSE;
 
-        target = cairo_image_surface_create (CAIRO_FORMAT_RGB24,
-                                             1, 1);
-        cr = cairo_create  (target);
+        target = cairo_image_surface_create (CAIRO_FORMAT_RGB24, 1, 1);
+        cr = cairo_create (target);
 
-        draw = rsvg_drawing_ctx_new (cr,
-                                     dimensions.width, dimensions.height,
-                                     dimensions.em, dimensions.ex,
-                                     handle->priv->dpi_x, handle->priv->dpi_y,
-                                     handle->priv->defs,
-                                     handle->priv->is_testing);
+        draw = rsvg_handle_create_drawing_ctx (handle, cr, &dimensions);
 
         g_assert (sself != NULL);
         rsvg_drawing_ctx_add_node_and_ancestors_to_stack (draw, sself);
@@ -965,8 +968,7 @@ rsvg_handle_get_dimensions_sub (RsvgHandle * handle, RsvgDimensionData * dimensi
         rsvg_handle_cascade (handle);
         rsvg_drawing_ctx_draw_node_from_stack (draw, handle->priv->treebase, NULL, FALSE);
 
-        bbox = rsvg_drawing_ctx_get_bbox (draw);
-        rsvg_bbox_get_rect (bbox, NULL, &ink_rect);
+        rsvg_drawing_ctx_get_ink_rect (draw, &ink_rect);
         dimension_data->width = ink_rect.width;
         dimension_data->height = ink_rect.height;
 
@@ -1008,7 +1010,6 @@ rsvg_handle_get_position_sub (RsvgHandle * handle, RsvgPositionData * position_d
     RsvgDrawingCtx		*draw;
     RsvgNode			*node;
     RsvgDimensionData    dimensions;
-    RsvgBbox            *bbox;
     cairo_rectangle_t    ink_rect;
     cairo_surface_t		*target = NULL;
     cairo_t				*cr = NULL;
@@ -1045,22 +1046,16 @@ rsvg_handle_get_position_sub (RsvgHandle * handle, RsvgPositionData * position_d
         return FALSE;
 
     target = cairo_image_surface_create (CAIRO_FORMAT_RGB24, 1, 1);
-    cr = cairo_create  (target);
+    cr = cairo_create (target);
 
-    draw = rsvg_drawing_ctx_new (cr,
-                                 dimensions.width, dimensions.height,
-                                 dimensions.em, dimensions.ex,
-                                 handle->priv->dpi_x, handle->priv->dpi_y,
-                                 handle->priv->defs,
-                                 handle->priv->is_testing);
+    draw = rsvg_handle_create_drawing_ctx (handle, cr, &dimensions);
 
     g_assert (node != NULL);
     rsvg_drawing_ctx_add_node_and_ancestors_to_stack (draw, node);
 
     rsvg_handle_cascade (handle);
     rsvg_drawing_ctx_draw_node_from_stack (draw, handle->priv->treebase, NULL, FALSE);
-    bbox = rsvg_drawing_ctx_get_bbox (draw);
-    rsvg_bbox_get_rect (bbox, NULL, &ink_rect);
+    rsvg_drawing_ctx_get_ink_rect (draw, &ink_rect);
     rsvg_drawing_ctx_free (draw);
 
     position_data->x = ink_rect.x;
