@@ -46,6 +46,10 @@ typedef enum {
 G_GNUC_INTERNAL
 RsvgNode *rsvg_load_new_node (const char *element_name, RsvgNode *parent, RsvgPropertyBag *atts);
 
+/* Implemented in rust/src/node.rs */
+G_GNUC_INTERNAL
+void rsvg_node_register_in_defs(RsvgNode *node, RsvgDefs *defs);
+
 struct RsvgLoad {
     RsvgHandle *handle;
     gboolean unlimited_size;
@@ -311,7 +315,7 @@ node_set_atts (RsvgNode *node,
         rsvg_parse_style_attrs (handle, node, element_name, atts);
     }
 
-    rsvg_node_set_overriden_properties (node);
+    rsvg_node_set_overridden_properties (node);
 }
 
 static void
@@ -943,6 +947,8 @@ sax_processing_instruction_cb (void *user_data, const xmlChar * target, const xm
                 }
             }
 
+            rsvg_property_bag_iter_end (iter);
+
             if ((!alternate || strcmp (alternate, "no") != 0)
                 && type && strcmp (type, "text/css") == 0
                 && href) {
