@@ -181,9 +181,6 @@ typedef struct {
     gboolean active;
 } RsvgViewBox;
 
-/* Opaque; defined in rsvg_internals/src/bbox.rs */
-typedef struct RsvgBbox RsvgBbox;
-
 /* Keep this in sync with rust/src/length.rs:LengthUnit */
 typedef enum {
     LENGTH_UNIT_DEFAULT,
@@ -327,11 +324,7 @@ void rsvg_node_set_atts (RsvgNode *node, RsvgHandle *handle, RsvgPropertyBag att
 
 /* Implemented in rust/src/node.rs */
 G_GNUC_INTERNAL
-void rsvg_node_set_overriden_properties (RsvgNode *node);
-
-/* Implemented in rust/src/node.rs */
-G_GNUC_INTERNAL
-void rsvg_node_draw (RsvgNode *node, RsvgComputedValues *parent_values, RsvgDrawingCtx *draw, int dominate, gboolean clipping);
+void rsvg_node_set_overridden_properties (RsvgNode *node);
 
 /* Implemented in rust/src/node.rs */
 G_GNUC_INTERNAL
@@ -422,28 +415,6 @@ GdkPixbuf *rsvg_cairo_surface_to_pixbuf (cairo_surface_t *surface);
 G_GNUC_INTERNAL
 cairo_surface_t *rsvg_cairo_surface_new_from_href (RsvgHandle *handle, const char *href, GError ** error);
 
-/* Implemented in rust/src/bbox.rs */
-G_GNUC_INTERNAL
-RsvgBbox *rsvg_bbox_new (cairo_matrix_t *matrix, cairo_rectangle_t *rect, cairo_rectangle_t *ink_rect);
-
-/* Implemented in rust/src/bbox.rs */
-void rsvg_bbox_free (RsvgBbox *bbox);
-
-/* Implemented in rust/src/bbox.rs */
-RsvgBbox *rsvg_bbox_clone (RsvgBbox *bbox);
-
-/* Implemented in rust/src/bbox.rs */
-G_GNUC_INTERNAL
-void rsvg_bbox_insert (RsvgBbox *dst, RsvgBbox *src);
-
-/* Implemented in rust/src/bbox.rs */
-G_GNUC_INTERNAL
-void rsvg_bbox_clip (RsvgBbox *dst, RsvgBbox *src);
-
-/* Implemented in rust/src/bbox.rs */
-G_GNUC_INTERNAL
-void rsvg_bbox_get_rect (RsvgBbox *bbox, cairo_rectangle_t *rect, cairo_rectangle_t *ink_rect);
-
 /* This is implemented in rust/src/length.rs */
 G_GNUC_INTERNAL
 double rsvg_length_normalize (const RsvgLength *length, RsvgComputedValues *values, RsvgDrawingCtx * ctx);
@@ -459,6 +430,38 @@ double rsvg_length_hand_normalize (const RsvgLength *length,
 G_GNUC_INTERNAL
 RsvgLength rsvg_length_parse (const char *str, LengthDir dir);
 
+/* Defined in rsvg_internals/src/drawing_ctx.rs */
+G_GNUC_INTERNAL
+RsvgDrawingCtx *rsvg_drawing_ctx_new (cairo_t *cr,
+                                      guint width,
+                                      guint height,
+                                      double vb_width,
+                                      double vb_height,
+                                      double dpi_x,
+                                      double dpi_y,
+                                      RsvgDefs *defs,
+                                      gboolean testing);
+
+/* Defined in rsvg_internals/src/drawing_ctx.rs */
+G_GNUC_INTERNAL
+void rsvg_drawing_ctx_free (RsvgDrawingCtx *draw_ctx);
+
+/* Defined in rsvg_internals/src/drawing_ctx.rs */
+G_GNUC_INTERNAL
+void rsvg_drawing_ctx_add_node_and_ancestors_to_stack (RsvgDrawingCtx *draw_ctx,
+                                                       RsvgNode        *node);
+
+/* Defined in rsvg_internals/src/drawing_ctx.rs */
+G_GNUC_INTERNAL
+void rsvg_drawing_ctx_draw_node_from_stack (RsvgDrawingCtx *ctx,
+                                            RsvgNode *node,
+                                            RsvgNode *cascade_from_node,
+                                            gboolean clipping);
+
+/* Defined in rsvg_internals/src/drawing_ctx.rs */
+G_GNUC_INTERNAL
+void rsvg_drawing_ctx_get_ink_rect (RsvgDrawingCtx *ctx, cairo_rectangle_t *ink_rect);
+
 /* Implemented in rust/src/node.rs */
 G_GNUC_INTERNAL
 void rsvg_root_node_cascade(RsvgNode *node);
