@@ -1,5 +1,5 @@
 use cairo;
-use cairo::MatrixTrait;
+use cairo::{MatrixTrait, Pattern};
 use cairo_sys;
 use glib::translate::*;
 use pango::{self, ContextExt, LayoutExt};
@@ -7,7 +7,7 @@ use pango_sys;
 use pangocairo;
 
 use bbox::BoundingBox;
-use drawing_ctx::{self, RsvgDrawingCtx};
+use drawing_ctx::DrawingCtx;
 use float_eq_cairo::ApproxEqCairo;
 use length::Dasharray;
 use paint_server;
@@ -25,8 +25,8 @@ use state::{
     TextRendering,
 };
 
-fn set_affine_on_cr(draw_ctx: *mut RsvgDrawingCtx, cr: &cairo::Context) {
-    let (x0, y0) = drawing_ctx::get_offset(draw_ctx);
+fn set_affine_on_cr(draw_ctx: &DrawingCtx, cr: &cairo::Context) {
+    let (x0, y0) = draw_ctx.get_offset();
 
     let affine = cr.get_matrix();
 
@@ -42,12 +42,12 @@ fn set_affine_on_cr(draw_ctx: *mut RsvgDrawingCtx, cr: &cairo::Context) {
 }
 
 pub fn draw_path_builder(
-    draw_ctx: *mut RsvgDrawingCtx,
+    draw_ctx: &mut DrawingCtx,
     values: &ComputedValues,
     builder: &PathBuilder,
     clipping: bool,
 ) {
-    let cr = drawing_ctx::get_cairo_context(draw_ctx);
+    let cr = draw_ctx.get_cairo_context();
 
     set_affine_on_cr(draw_ctx, &cr);
 
@@ -62,7 +62,7 @@ pub fn draw_path_builder(
     }
 }
 
-fn stroke_and_fill(cr: &cairo::Context, draw_ctx: *mut RsvgDrawingCtx, values: &ComputedValues) {
+fn stroke_and_fill(cr: &cairo::Context, draw_ctx: &mut DrawingCtx, values: &ComputedValues) {
     cr.set_antialias(cairo::Antialias::from(values.shape_rendering));
 
     setup_cr_for_stroke(cr, draw_ctx, values);
@@ -72,7 +72,7 @@ fn stroke_and_fill(cr: &cairo::Context, draw_ctx: *mut RsvgDrawingCtx, values: &
     // rendering context to have an updated bbox; for example, for the
     // coordinate system in patterns.
     let bbox = compute_stroke_and_fill_box(cr, values);
-    drawing_ctx::insert_bbox(draw_ctx, &bbox);
+    draw_ctx.insert_bbox(&bbox);
 
     let current_color = &values.color.0;
 
@@ -198,11 +198,7 @@ impl From<TextRendering> for cairo::Antialias {
     }
 }
 
-fn setup_cr_for_stroke(
-    cr: &cairo::Context,
-    draw_ctx: *const RsvgDrawingCtx,
-    values: &ComputedValues,
-) {
+fn setup_cr_for_stroke(cr: &cairo::Context, draw_ctx: &DrawingCtx, values: &ComputedValues) {
     cr.set_line_width(values.stroke_width.0.normalize(values, draw_ctx));
     cr.set_miter_limit(values.stroke_miterlimit.0);
     cr.set_line_cap(cairo::LineCap::from(values.stroke_line_cap));
@@ -238,28 +234,6 @@ fn path_extents(cr: &cairo::Context) -> (f64, f64, f64, f64) {
     (x1, y1, x2, y2)
 }
 
-fn bbox_from_extents(
-    affine: &cairo::Matrix,
-    (x1, y1, x2, y2): (f64, f64, f64, f64),
-    ink: bool,
-) -> BoundingBox {
-    let mut bb = BoundingBox::new(affine);
-    let rect = cairo::Rectangle {
-        x: x1,
-        y: y1,
-        width: x2 - x1,
-        height: y2 - y1,
-    };
-
-    if ink {
-        bb.ink_rect = Some(rect);
-    } else {
-        bb.rect = Some(rect);
-    }
-
-    bb
-}
-
 fn compute_stroke_and_fill_box(cr: &cairo::Context, values: &ComputedValues) -> BoundingBox {
     let affine = cr.get_matrix();
 
@@ -279,19 +253,19 @@ fn compute_stroke_and_fill_box(cr: &cairo::Context, values: &ComputedValues) ->
     // paths for the icon's shape.  We need to be able to compute the bounding
     // rectangle's extents, even when it has no fill nor stroke.
 
-    let fb = bbox_from_extents(&affine, cr.fill_extents(), true);
+    let fb = BoundingBox::new(&affine).with_ink_extents(cr.fill_extents());
     bbox.insert(&fb);
 
     // Bounding box for stroke
 
     if values.stroke.0 != PaintServer::None {
-        let sb = bbox_from_extents(&affine, cr.stroke_extents(), true);
+        let sb = BoundingBox::new(&affine).with_ink_extents(cr.stroke_extents());
         bbox.insert(&sb);
     }
 
     // objectBoundingBox
 
-    let ob = bbox_from_extents(&affine, path_extents(cr), false);
+    let ob = BoundingBox::new(&affine).with_extents(path_extents(cr));
     bbox.insert(&ob);
 
     // restore tolerance
@@ -302,7 +276,7 @@ fn compute_stroke_and_fill_box(cr: &cairo::Context, values: &ComputedValues) ->
 }
 
 pub fn draw_pango_layout(
-    draw_ctx: *mut RsvgDrawingCtx,
+    draw_ctx: &mut DrawingCtx,
     values: &ComputedValues,
     layout: &pango::Layout,
     x: f64,
@@ -317,7 +291,7 @@ pub fn draw_pango_layout(
         return;
     }
 
-    let cr = drawing_ctx::get_cairo_context(draw_ctx);
+    let cr = draw_ctx.get_cairo_context();
     cr.save();
 
     set_affine_on_cr(draw_ctx, &cr);
@@ -327,7 +301,7 @@ pub fn draw_pango_layout(
     let bbox = compute_text_bbox(&ink, x, y, &affine, gravity);
 
     if !clipping {
-        drawing_ctx::insert_bbox(draw_ctx, &bbox);
+        draw_ctx.insert_bbox(&bbox);
     }
 
     cr.set_antialias(cairo::Antialias::from(values.text_rendering));
@@ -379,9 +353,9 @@ pub fn draw_pango_layout(
         pangocairo::functions::layout_path(&cr, layout);
 
         if !clipping {
-            let ib = bbox_from_extents(&affine, cr.stroke_extents(), true);
+            let ib = BoundingBox::new(&affine).with_ink_extents(cr.stroke_extents());
             cr.stroke();
-            drawing_ctx::insert_bbox(draw_ctx, &ib);
+            draw_ctx.insert_bbox(&ib);
         }
     }
 
@@ -432,7 +406,7 @@ fn compute_text_bbox(
 }
 
 pub fn draw_surface(
-    draw_ctx: *mut RsvgDrawingCtx,
+    draw_ctx: &mut DrawingCtx,
     values: &ComputedValues,
     surface: &cairo::ImageSurface,
     x: f64,
@@ -445,7 +419,7 @@ pub fn draw_surface(
         return;
     }
 
-    let cr = drawing_ctx::get_cairo_context(draw_ctx);
+    let cr = draw_ctx.get_cairo_context();
 
     let affine = cr.get_matrix();
 
@@ -461,13 +435,13 @@ pub fn draw_surface(
     let width = f64::from(width);
     let height = f64::from(height);
 
-    let mut bbox = BoundingBox::new(&affine);
-    bbox.rect = Some(cairo::Rectangle {
+    // This is the target bbox after drawing.
+    let bbox = BoundingBox::new(&affine).with_rect(Some(cairo::Rectangle {
         x,
         y,
-        width,
-        height,
-    });
+        width: w,
+        height: h,
+    }));
 
     set_affine_on_cr(draw_ctx, &cr);
     cr.scale(w / width, h / height);
@@ -476,17 +450,33 @@ pub fn draw_surface(
 
     cr.set_operator(cairo::Operator::from(values.comp_op));
 
-    cr.set_source_surface(&surface, x, y);
+    // We need to set extend appropriately, so can't use cr.set_source_surface().
+    //
+    // If extend is left at its default value (None), then bilinear scaling uses transparency
+    // outside of the image producing incorrect results. For example, in
+    // svg1.1/filters-blend-01-b.svg there's a completely opaque 100×1 image of a gradient scaled
+    // to 100×98 which ends up transparent almost everywhere without this fix (which it shouldn't).
+    let ptn = cairo::SurfacePattern::create(&surface);
+    let mut matrix = cairo::Matrix::identity();
+    matrix.translate(-x, -y);
+    ptn.set_matrix(matrix);
+    ptn.set_extend(cairo::Extend::Pad);
+    cr.set_source(&ptn);
+
+    // Clip is needed due to extend being set to pad.
+    cr.rectangle(x, y, width, height);
+    cr.clip();
+
     cr.paint();
 
     cr.restore();
 
-    drawing_ctx::insert_bbox(draw_ctx, &bbox);
+    draw_ctx.insert_bbox(&bbox);
 }
 
 /// Adds a clipping rectangle to the curent Cairo context
-pub fn add_clipping_rect(draw_ctx: *mut RsvgDrawingCtx, x: f64, y: f64, w: f64, h: f64) {
-    let cr = drawing_ctx::get_cairo_context(draw_ctx);
+pub fn add_clipping_rect(draw_ctx: &mut DrawingCtx, x: f64, y: f64, w: f64, h: f64) {
+    let cr = draw_ctx.get_cairo_context();
 
     let save_affine = cr.get_matrix();
 
