@@ -1,20 +1,26 @@
 use std::cell::{Cell, RefCell};
 
 use cairo::{self, ImageSurface};
+use cssparser::{CowRcStr, Parser, Token};
 
 use attributes::Attribute;
+use drawing_ctx::DrawingCtx;
 use error::{AttributeError, NodeError};
 use handle::RsvgHandle;
 use node::{NodeResult, NodeTrait, RsvgCNodeImpl, RsvgNode};
 use parsers::{self, parse, Parse};
 use property_bag::PropertyBag;
-use srgb::{linearize_surface, unlinearize_surface};
+use surface_utils::{
+    iterators::Pixels,
+    shared_surface::SharedImageSurface,
+    ImageSurfaceDataExt,
+    Pixel,
+};
 use util::clamp;
 
 use super::context::{FilterContext, FilterOutput, FilterResult};
 use super::input::Input;
-use super::iterators::{ImageSurfaceDataExt, ImageSurfaceDataShared, Pixel, Pixels};
-use super::{get_surface, Filter, FilterError, PrimitiveWithInput};
+use super::{make_result, Filter, FilterError, PrimitiveWithInput};
 
 /// Enumeration of the possible compositing operations.
 #[derive(Debug, Clone, Copy, Eq, PartialEq, Hash)]
@@ -66,9 +72,9 @@ impl NodeTrait for Composite {
         for (_key, attr, value) in pbag.iter() {
             match attr {
                 Attribute::In2 => {
-                    self.in2.replace(Some(parse("in2", value, (), None)?));
+                    self.in2.replace(Some(Input::parse(Attribute::In2, value)?));
                 }
-                Attribute::Operator => self.operator.set(parse("operator", value, (), None)?),
+                Attribute::Operator => self.operator.set(parse("operator", value, ())?),
                 Attribute::K1 => self
                     .k1
                     .set(parsers::number(value).map_err(|err| NodeError::parse_error(attr, err))?),
@@ -95,25 +101,26 @@ impl NodeTrait for Composite {
 }
 
 impl Filter for Composite {
-    fn render(&self, _node: &RsvgNode, ctx: &FilterContext) -> Result<FilterResult, FilterError> {
-        let bounds = self.base.get_bounds(ctx);
-
-        let input_surface = get_surface(self.base.get_input(ctx))?;
-        let input_2_surface = get_surface(ctx.get_input(self.in2.borrow().as_ref()))?;
-
-        // It's important to linearize sRGB before doing any blending, since otherwise the colors
-        // will be darker than they should be.
-        let input_surface =
-            linearize_surface(&input_surface, bounds).map_err(FilterError::BadInputSurfaceStatus)?;
+    fn render(
+        &self,
+        _node: &RsvgNode,
+        ctx: &FilterContext,
+        draw_ctx: &mut DrawingCtx,
+    ) -> Result<FilterResult, FilterError> {
+        let input = make_result(self.base.get_input(ctx, draw_ctx))?;
+        let input_2 = make_result(ctx.get_input(draw_ctx, self.in2.borrow().as_ref()))?;
+        let bounds = self
+            .base
+            .get_bounds(ctx)
+            .add_input(&input)
+            .add_input(&input_2)
+            .into_irect(draw_ctx);
 
         let output_surface = if self.operator.get() == Operator::Arithmetic {
-            let input_data = ImageSurfaceDataShared::new(&input_surface)?;
-            let input_2_data = ImageSurfaceDataShared::new(&input_2_surface)?;
-
             let mut output_surface = ImageSurface::create(
                 cairo::Format::ARgb32,
-                input_data.width as i32,
-                input_data.height as i32,
+                input.surface().width(),
+                input.surface().height(),
             ).map_err(FilterError::OutputSurfaceCreation)?;
 
             let output_stride = output_surface.get_stride() as usize;
@@ -125,8 +132,8 @@ impl Filter for Composite {
                 let k3 = self.k3.get();
                 let k4 = self.k4.get();
 
-                for (x, y, pixel, pixel_2) in Pixels::new(input_data, bounds)
-                    .map(|(x, y, p)| (x, y, p, input_2_data.get_pixel(x, y)))
+                for (x, y, pixel, pixel_2) in Pixels::new(input.surface(), bounds)
+                    .map(|(x, y, p)| (x, y, p, input_2.surface().get_pixel(x, y)))
                 {
                     let i1a = f64::from(pixel.a) / 255f64;
                     let i2a = f64::from(pixel_2.a) / 255f64;
@@ -160,8 +167,10 @@ impl Filter for Composite {
 
             output_surface
         } else {
-            let output_surface = linearize_surface(&input_2_surface, bounds)
-                .map_err(FilterError::BadInputSurfaceStatus)?;
+            let output_surface = input_2
+                .surface()
+                .copy_surface(bounds)
+                .map_err(FilterError::OutputSurfaceCreation)?;
 
             let cr = cairo::Context::new(&output_surface);
             cr.rectangle(
@@ -172,40 +181,51 @@ impl Filter for Composite {
             );
             cr.clip();
 
-            cr.set_source_surface(&input_surface, 0f64, 0f64);
+            input.surface().set_as_source_surface(&cr, 0f64, 0f64);
             cr.set_operator(self.operator.get().into());
             cr.paint();
 
             output_surface
         };
 
-        let output_surface = unlinearize_surface(&output_surface, bounds)
-            .map_err(FilterError::OutputSurfaceCreation)?;
-
         Ok(FilterResult {
             name: self.base.result.borrow().clone(),
             output: FilterOutput {
-                surface: output_surface,
+                surface: SharedImageSurface::new(output_surface).unwrap(),
                 bounds,
             },
         })
     }
+
+    #[inline]
+    fn is_affected_by_color_interpolation_filters() -> bool {
+        true
+    }
 }
 
 impl Parse for Operator {
     type Data = ();
     type Err = AttributeError;
 
-    fn parse(s: &str, _data: Self::Data) -> Result<Self, Self::Err> {
-        match s {
-            "over" => Ok(Operator::Over),
-            "in" => Ok(Operator::In),
-            "out" => Ok(Operator::Out),
-            "atop" => Ok(Operator::Atop),
-            "xor" => Ok(Operator::Xor),
-            "arithmetic" => Ok(Operator::Arithmetic),
-            _ => Err(AttributeError::Value("invalid operator value".to_string())),
-        }
+    fn parse(parser: &mut Parser, _data: Self::Data) -> Result<Self, Self::Err> {
+        let loc = parser.current_source_location();
+
+        parser
+            .expect_ident()
+            .and_then(|cow| match cow.as_ref() {
+                "over" => Ok(Operator::Over),
+                "in" => Ok(Operator::In),
+                "out" => Ok(Operator::Out),
+                "atop" => Ok(Operator::Atop),
+                "xor" => Ok(Operator::Xor),
+                "arithmetic" => Ok(Operator::Arithmetic),
+                _ => Err(
+                    loc.new_basic_unexpected_token_error(Token::Ident(CowRcStr::from(
+                        cow.as_ref().to_string(),
+                    ))),
+                ),
+            })
+            .map_err(|_| AttributeError::Value("invalid operator value".to_string()))
     }
 }
 
