@@ -1,10 +1,8 @@
-use cssparser::{Parser, ParserInput, Token};
+use cssparser::{Parser, Token};
 use libc;
-use regex::Regex;
 use std::f64::consts::*;
 
-use drawing_ctx;
-use drawing_ctx::RsvgDrawingCtx;
+use drawing_ctx::{DrawingCtx, RsvgDrawingCtx};
 use error::*;
 use parsers::Parse;
 use parsers::ParseError;
@@ -99,7 +97,7 @@ pub extern "C" fn rsvg_length_parse(string: *const libc::c_char, dir: LengthDir)
     let my_string = unsafe { utf8_cstr(string) };
 
     // FIXME: this ignores errors; propagate them upstream
-    RsvgLength::parse(my_string, dir).unwrap_or_else(|_| RsvgLength::default())
+    RsvgLength::parse_str(my_string, dir).unwrap_or_else(|_| RsvgLength::default())
 }
 
 // https://www.w3.org/TR/SVG/types.html#DataTypeLength
@@ -123,11 +121,8 @@ impl Parse for RsvgLength {
     type Data = LengthDir;
     type Err = AttributeError;
 
-    fn parse(string: &str, dir: LengthDir) -> Result<RsvgLength, AttributeError> {
-        let mut input = ParserInput::new(string);
-        let mut parser = Parser::new(&mut input);
-
-        let length = RsvgLength::from_cssparser(&mut parser, dir)?;
+    fn parse(parser: &mut Parser, dir: LengthDir) -> Result<RsvgLength, AttributeError> {
+        let length = RsvgLength::from_cssparser(parser, dir)?;
 
         parser.expect_exhausted().map_err(|_| make_err())?;
 
@@ -154,12 +149,12 @@ impl RsvgLength {
         }
     }
 
-    pub fn normalize(&self, values: &ComputedValues, draw_ctx: *const RsvgDrawingCtx) -> f64 {
+    pub fn normalize(&self, values: &ComputedValues, draw_ctx: &DrawingCtx) -> f64 {
         match self.unit {
             LengthUnit::Default => self.length,
 
             LengthUnit::Percent => {
-                let (width, height) = drawing_ctx::get_view_box_size(draw_ctx);
+                let (width, height) = draw_ctx.get_view_box_size();
 
                 match self.dir {
                     LengthDir::Horizontal => self.length * width,
@@ -180,6 +175,13 @@ impl RsvgLength {
         }
     }
 
+    /// Returns the raw length after asserting units are either default or percent.
+    #[inline]
+    pub fn get_unitless(&self) -> f64 {
+        assert!(self.unit == LengthUnit::Default || self.unit == LengthUnit::Percent);
+        self.length
+    }
+
     pub fn hand_normalize(
         &self,
         pixels_per_inch: f64,
@@ -201,7 +203,10 @@ impl RsvgLength {
         }
     }
 
-    fn from_cssparser(parser: &mut Parser, dir: LengthDir) -> Result<RsvgLength, AttributeError> {
+    pub fn from_cssparser(
+        parser: &mut Parser,
+        dir: LengthDir,
+    ) -> Result<RsvgLength, AttributeError> {
         let length = {
             let token = parser.next().map_err(|_| {
                 AttributeError::Parse(ParseError::new(
@@ -312,8 +317,8 @@ impl RsvgLength {
     }
 }
 
-fn font_size_from_inch(length: f64, dir: LengthDir, draw_ctx: *const RsvgDrawingCtx) -> f64 {
-    let (dpi_x, dpi_y) = drawing_ctx::get_dpi(draw_ctx);
+fn font_size_from_inch(length: f64, dir: LengthDir, draw_ctx: &DrawingCtx) -> f64 {
+    let (dpi_x, dpi_y) = draw_ctx.get_dpi();
 
     match dir {
         LengthDir::Horizontal => length * dpi_x,
@@ -322,7 +327,7 @@ fn font_size_from_inch(length: f64, dir: LengthDir, draw_ctx: *const RsvgDrawing
     }
 }
 
-fn font_size_from_values(values: &ComputedValues, draw_ctx: *const RsvgDrawingCtx) -> f64 {
+fn font_size_from_values(values: &ComputedValues, draw_ctx: &DrawingCtx) -> f64 {
     let v = &values.font_size.0;
 
     match v.unit {
@@ -364,52 +369,33 @@ impl Parse for Dasharray {
     type Data = ();
     type Err = AttributeError;
 
-    fn parse(s: &str, _: Self::Data) -> Result<Dasharray, AttributeError> {
-        match s.trim() {
-            "none" => Ok(Dasharray::None),
-            _ => Ok(Dasharray::Array(parse_dash_array(s)?)),
+    fn parse(parser: &mut Parser, _: Self::Data) -> Result<Dasharray, AttributeError> {
+        if parser.try(|p| p.expect_ident_matching("none")).is_ok() {
+            Ok(Dasharray::None)
+        } else {
+            Ok(Dasharray::Array(parse_dash_array(parser)?))
         }
     }
 }
 
 // This does not handle "inherit" or "none" state, the caller is responsible for that.
-fn parse_dash_array(s: &str) -> Result<Vec<RsvgLength>, AttributeError> {
-    lazy_static!{
-        // The unwrap here is fine AS LONG the regex query is valid.
-        static ref COMMAS: Regex = Regex::new(r",\s*,").unwrap();
-    };
+fn parse_dash_array(parser: &mut Parser) -> Result<Vec<RsvgLength>, AttributeError> {
+    let mut dasharray = Vec::new();
 
-    let s = s.trim();
+    loop {
+        dasharray.push(
+            RsvgLength::from_cssparser(parser, LengthDir::Both)
+                .and_then(RsvgLength::check_nonnegative)?,
+        );
 
-    if s.is_empty() {
-        return Err(AttributeError::Parse(ParseError::new("empty string")));
-    }
-
-    // Read the last character, if it's a comma return an Error.
-    if let Some(c) = s.chars().last() {
-        if c == ',' {
-            return Err(AttributeError::Parse(ParseError::new("trailing comma")));
+        if parser.is_exhausted() {
+            break;
+        } else if parser.try(|p| p.expect_comma()).is_ok() {
+            continue;
         }
     }
 
-    // Commas must be followed by a value.
-    if COMMAS.is_match(s) {
-        return Err(AttributeError::Parse(ParseError::new(
-            "expected number, found comma",
-        )));
-    }
-
-    // Values can be comma or whitespace separated.
-    s.split(',') // split at comma
-        // split at whitespace
-        .flat_map(|slice| slice.split_whitespace())
-        // parse it into an RsvgLength
-        .map(|d| RsvgLength::parse(d, LengthDir::Both)
-             .and_then(|l| l.check_nonnegative()))
-        // collect into a Result<Vec<T>, E>.
-        // it will short-circuit iteslf upon the first error encountered
-        // like if you returned from a for-loop
-        .collect::<Result<Vec<_>, _>>()
+    Ok(dasharray)
 }
 
 #[no_mangle]
@@ -424,6 +410,9 @@ pub extern "C" fn rsvg_length_normalize(
     assert!(!values.is_null());
     let values = unsafe { &*values };
 
+    assert!(!draw_ctx.is_null());
+    let draw_ctx = unsafe { &*(draw_ctx as *const DrawingCtx) };
+
     length.normalize(values, draw_ctx)
 }
 
@@ -448,7 +437,7 @@ mod tests {
     #[test]
     fn parses_default() {
         assert_eq!(
-            RsvgLength::parse("42", LengthDir::Horizontal),
+            RsvgLength::parse_str("42", LengthDir::Horizontal),
             Ok(RsvgLength::new(
                 42.0,
                 LengthUnit::Default,
@@ -457,7 +446,7 @@ mod tests {
         );
 
         assert_eq!(
-            RsvgLength::parse("-42px", LengthDir::Horizontal),
+            RsvgLength::parse_str("-42px", LengthDir::Horizontal),
             Ok(RsvgLength::new(
                 -42.0,
                 LengthUnit::Default,
@@ -469,7 +458,7 @@ mod tests {
     #[test]
     fn parses_percent() {
         assert_eq!(
-            RsvgLength::parse("50.0%", LengthDir::Horizontal),
+            RsvgLength::parse_str("50.0%", LengthDir::Horizontal),
             Ok(RsvgLength::new(
                 0.5,
                 LengthUnit::Percent,
@@ -481,7 +470,7 @@ mod tests {
     #[test]
     fn parses_font_em() {
         assert_eq!(
-            RsvgLength::parse("22.5em", LengthDir::Vertical),
+            RsvgLength::parse_str("22.5em", LengthDir::Vertical),
             Ok(RsvgLength::new(
                 22.5,
                 LengthUnit::FontEm,
@@ -493,7 +482,7 @@ mod tests {
     #[test]
     fn parses_font_ex() {
         assert_eq!(
-            RsvgLength::parse("22.5ex", LengthDir::Vertical),
+            RsvgLength::parse_str("22.5ex", LengthDir::Vertical),
             Ok(RsvgLength::new(
                 22.5,
                 LengthUnit::FontEx,
@@ -505,27 +494,27 @@ mod tests {
     #[test]
     fn parses_physical_units() {
         assert_eq!(
-            RsvgLength::parse("72pt", LengthDir::Both),
+            RsvgLength::parse_str("72pt", LengthDir::Both),
             Ok(RsvgLength::new(1.0, LengthUnit::Inch, LengthDir::Both))
         );
 
         assert_eq!(
-            RsvgLength::parse("-22.5in", LengthDir::Both),
+            RsvgLength::parse_str("-22.5in", LengthDir::Both),
             Ok(RsvgLength::new(-22.5, LengthUnit::Inch, LengthDir::Both))
         );
 
         assert_eq!(
-            RsvgLength::parse("-254cm", LengthDir::Both),
+            RsvgLength::parse_str("-254cm", LengthDir::Both),
             Ok(RsvgLength::new(-100.0, LengthUnit::Inch, LengthDir::Both))
         );
 
         assert_eq!(
-            RsvgLength::parse("254mm", LengthDir::Both),
+            RsvgLength::parse_str("254mm", LengthDir::Both),
             Ok(RsvgLength::new(10.0, LengthUnit::Inch, LengthDir::Both))
         );
 
         assert_eq!(
-            RsvgLength::parse("60pc", LengthDir::Both),
+            RsvgLength::parse_str("60pc", LengthDir::Both),
             Ok(RsvgLength::new(10.0, LengthUnit::Inch, LengthDir::Both))
         );
     }
@@ -533,7 +522,7 @@ mod tests {
     #[test]
     fn parses_relative_larger() {
         assert_eq!(
-            RsvgLength::parse("larger", LengthDir::Vertical),
+            RsvgLength::parse_str("larger", LengthDir::Vertical),
             Ok(RsvgLength::new(
                 0.0,
                 LengthUnit::RelativeLarger,
@@ -545,7 +534,7 @@ mod tests {
     #[test]
     fn parses_relative_smaller() {
         assert_eq!(
-            RsvgLength::parse("smaller", LengthDir::Vertical),
+            RsvgLength::parse_str("smaller", LengthDir::Vertical),
             Ok(RsvgLength::new(
                 0.0,
                 LengthUnit::RelativeSmaller,
@@ -566,7 +555,7 @@ mod tests {
         // enforce a particular sequence.
 
         for name in names {
-            let length = RsvgLength::parse(name, LengthDir::Both).unwrap();
+            let length = RsvgLength::parse_str(name, LengthDir::Both).unwrap();
 
             assert_eq!(length.unit, LengthUnit::Inch);
             assert_eq!(length.dir, LengthDir::Both);
@@ -581,12 +570,12 @@ mod tests {
 
     #[test]
     fn empty_length_yields_error() {
-        assert!(is_parse_error(&RsvgLength::parse("", LengthDir::Both)));
+        assert!(is_parse_error(&RsvgLength::parse_str("", LengthDir::Both)));
     }
 
     #[test]
     fn invalid_unit_yields_error() {
-        assert!(is_parse_error(&RsvgLength::parse(
+        assert!(is_parse_error(&RsvgLength::parse_str(
             "8furlong",
             LengthDir::Both
         )));
@@ -597,7 +586,7 @@ mod tests {
         // FIXME: this is intended to test the (absence of) the "larger" et al values.
         // Since they really be in FontSize, not RsvgLength, we should remember
         // to move this test to that type later.
-        assert!(is_parse_error(&RsvgLength::parse(
+        assert!(is_parse_error(&RsvgLength::parse_str(
             "furlong",
             LengthDir::Both
         )));
@@ -606,80 +595,84 @@ mod tests {
     #[test]
     fn check_nonnegative_works() {
         assert!(
-            RsvgLength::parse("0", LengthDir::Both)
+            RsvgLength::parse_str("0", LengthDir::Both)
                 .and_then(|l| l.check_nonnegative())
                 .is_ok()
         );
         assert!(
-            RsvgLength::parse("-10", LengthDir::Both)
+            RsvgLength::parse_str("-10", LengthDir::Both)
                 .and_then(|l| l.check_nonnegative())
                 .is_err()
         );
     }
 
+    fn parse_dash_array_str(s: &str) -> Result<Dasharray, AttributeError> {
+        Dasharray::parse_str(s, ())
+    }
+
     #[test]
     fn parses_dash_array() {
         // helper to cut down boilderplate
-        let length_parse = |s| RsvgLength::parse(s, LengthDir::Both).unwrap();
+        let length_parse = |s| RsvgLength::parse_str(s, LengthDir::Both).unwrap();
 
-        let expected = vec![
+        let expected = Dasharray::Array(vec![
             length_parse("1"),
             length_parse("2in"),
             length_parse("3"),
             length_parse("4%"),
-        ];
+        ]);
 
-        let sample_1 = vec![length_parse("10"), length_parse("6")];
-        let sample_2 = vec![length_parse("5"), length_parse("5"), length_parse("20")];
+        let sample_1 = Dasharray::Array(vec![length_parse("10"), length_parse("6")]);
 
-        let sample_3 = vec![
+        let sample_2 = Dasharray::Array(vec![
+            length_parse("5"),
+            length_parse("5"),
+            length_parse("20"),
+        ]);
+
+        let sample_3 = Dasharray::Array(vec![
             length_parse("10px"),
             length_parse("20px"),
             length_parse("20px"),
-        ];
+        ]);
 
-        let sample_4 = vec![
+        let sample_4 = Dasharray::Array(vec![
             length_parse("25"),
             length_parse("5"),
             length_parse("5"),
             length_parse("5"),
-        ];
+        ]);
 
-        let sample_5 = vec![length_parse("3.1415926"), length_parse("8")];
-        let sample_6 = vec![length_parse("5"), length_parse("3.14")];
-        let sample_7 = vec![length_parse("2")];
+        let sample_5 = Dasharray::Array(vec![length_parse("3.1415926"), length_parse("8")]);
+        let sample_6 = Dasharray::Array(vec![length_parse("5"), length_parse("3.14")]);
+        let sample_7 = Dasharray::Array(vec![length_parse("2")]);
 
-        assert_eq!(parse_dash_array("1 2in,3 4%").unwrap(), expected);
-        assert_eq!(parse_dash_array("10,6").unwrap(), sample_1);
-        assert_eq!(parse_dash_array("5,5,20").unwrap(), sample_2);
-        assert_eq!(parse_dash_array("10px 20px 20px").unwrap(), sample_3);
-        assert_eq!(parse_dash_array("25  5 , 5 5").unwrap(), sample_4);
-        assert_eq!(parse_dash_array("3.1415926,8").unwrap(), sample_5);
-        assert_eq!(parse_dash_array("5, 3.14").unwrap(), sample_6);
-        assert_eq!(parse_dash_array("2").unwrap(), sample_7);
+        assert_eq!(parse_dash_array_str("none").unwrap(), Dasharray::None);
+        assert_eq!(parse_dash_array_str("1 2in,3 4%").unwrap(), expected);
+        assert_eq!(parse_dash_array_str("10,6").unwrap(), sample_1);
+        assert_eq!(parse_dash_array_str("5,5,20").unwrap(), sample_2);
+        assert_eq!(parse_dash_array_str("10px 20px 20px").unwrap(), sample_3);
+        assert_eq!(parse_dash_array_str("25  5 , 5 5").unwrap(), sample_4);
+        assert_eq!(parse_dash_array_str("3.1415926,8").unwrap(), sample_5);
+        assert_eq!(parse_dash_array_str("5, 3.14").unwrap(), sample_6);
+        assert_eq!(parse_dash_array_str("2").unwrap(), sample_7);
 
         // Negative numbers
         assert_eq!(
-            parse_dash_array("20,40,-20"),
+            parse_dash_array_str("20,40,-20"),
             Err(AttributeError::Value(String::from(
                 "value must be non-negative"
             )))
         );
 
         // Empty dash_array
-        assert_eq!(
-            parse_dash_array(""),
-            Err(AttributeError::Parse(ParseError::new("empty string")))
-        );
-        assert_eq!(
-            parse_dash_array("\t  \n     "),
-            Err(AttributeError::Parse(ParseError::new("empty string")))
-        );
-        assert!(parse_dash_array(",,,").is_err());
-        assert!(parse_dash_array("10,  \t, 20 \n").is_err());
-        // No trailling commas allowed, parse error
-        assert!(parse_dash_array("10,").is_err());
+        assert!(parse_dash_array_str("").is_err());
+        assert!(parse_dash_array_str("\t  \n     ").is_err());
+        assert!(parse_dash_array_str(",,,").is_err());
+        assert!(parse_dash_array_str("10,  \t, 20 \n").is_err());
+        // No trailing commas allowed, parse error
+        assert!(parse_dash_array_str("10,").is_err());
         // A comma should be followed by a number
-        assert!(parse_dash_array("20,,10").is_err());
+        assert!(parse_dash_array_str("20,,10").is_err());
     }
 }
