@@ -24,35 +24,22 @@ extern crate downcast_rs;
 
 pub use attributes::rsvg_attribute_from_name;
 
-pub use bbox::{
-    rsvg_bbox_clip,
-    rsvg_bbox_clone,
-    rsvg_bbox_free,
-    rsvg_bbox_get_rect,
-    rsvg_bbox_insert,
-    rsvg_bbox_new,
-    RsvgBbox,
-};
-
 pub use cnode::{rsvg_rust_cnode_get_impl, rsvg_rust_cnode_new};
 
 pub use color::{rsvg_css_parse_color, ColorKind, ColorSpec};
 
 pub use drawing_ctx::{
+    rsvg_drawing_ctx_add_node_and_ancestors_to_stack,
     rsvg_drawing_ctx_draw_node_from_stack,
-    rsvg_drawing_ctx_transformed_image_bounding_box,
+    rsvg_drawing_ctx_free,
+    rsvg_drawing_ctx_get_ink_rect,
+    rsvg_drawing_ctx_new,
 };
 
 pub use filters::context::{
-    rsvg_filter_context_get_affine,
-    rsvg_filter_context_get_bg_surface,
     rsvg_filter_context_get_channelmap,
-    rsvg_filter_context_get_drawing_ctx,
     rsvg_filter_context_get_height,
-    rsvg_filter_context_get_lastresult,
-    rsvg_filter_context_get_node_being_filtered,
     rsvg_filter_context_get_paffine,
-    rsvg_filter_context_get_previous_result,
     rsvg_filter_context_get_source_surface,
     rsvg_filter_context_get_width,
     rsvg_filter_get_in,
@@ -85,7 +72,7 @@ pub use node::{
     rsvg_node_register_in_defs,
     rsvg_node_set_attribute_parse_error,
     rsvg_node_set_atts,
-    rsvg_node_set_overriden_properties,
+    rsvg_node_set_overridden_properties,
     rsvg_node_unref,
     rsvg_root_node_cascade,
 };
@@ -101,8 +88,6 @@ pub use property_bag::{
 };
 
 pub use state::{
-    rsvg_computed_values_get_flood_color_argb,
-    rsvg_computed_values_get_flood_opacity,
     rsvg_computed_values_get_lighting_color_argb,
     rsvg_parse_style_attrs,
     rsvg_state_free,
@@ -160,6 +145,7 @@ mod srgb;
 mod state;
 mod stop;
 mod structure;
+pub mod surface_utils;
 mod text;
 mod transform;
 mod unitinterval;
