@@ -5,7 +5,11 @@ use std::ptr;
 
 use attributes::Attribute;
 use clip_path::NodeClipPath;
+use filters::blend::Blend;
+use filters::component_transfer::{ComponentTransfer, FuncX};
 use filters::composite::Composite;
+use filters::flood::Flood;
+use filters::image::Image;
 use filters::merge::{Merge, MergeNode};
 use filters::node::NodeFilter;
 use filters::offset::Offset;
@@ -25,24 +29,12 @@ use util::utf8_cstr;
 
 #[allow(improper_ctypes)]
 extern "C" {
-    fn rsvg_new_filter_primitive_blend(
-        _: *const libc::c_char,
-        _: *const RsvgNode,
-        _: *const libc::c_char,
-        _: *const libc::c_char,
-    ) -> *const RsvgNode;
     fn rsvg_new_filter_primitive_color_matrix(
         _: *const libc::c_char,
         _: *const RsvgNode,
         _: *const libc::c_char,
         _: *const libc::c_char,
     ) -> *const RsvgNode;
-    fn rsvg_new_filter_primitive_component_transfer(
-        _: *const libc::c_char,
-        _: *const RsvgNode,
-        _: *const libc::c_char,
-        _: *const libc::c_char,
-    ) -> *const RsvgNode;
     fn rsvg_new_filter_primitive_convolve_matrix(
         _: *const libc::c_char,
         _: *const RsvgNode,
@@ -67,30 +59,12 @@ extern "C" {
         _: *const libc::c_char,
         _: *const libc::c_char,
     ) -> *const RsvgNode;
-    fn rsvg_new_filter_primitive_flood(
-        _: *const libc::c_char,
-        _: *const RsvgNode,
-        _: *const libc::c_char,
-        _: *const libc::c_char,
-    ) -> *const RsvgNode;
-    fn rsvg_new_node_component_transfer_function(
-        _: *const libc::c_char,
-        _: *const RsvgNode,
-        _: *const libc::c_char,
-        _: *const libc::c_char,
-    ) -> *const RsvgNode;
     fn rsvg_new_filter_primitive_gaussian_blur(
         _: *const libc::c_char,
         _: *const RsvgNode,
         _: *const libc::c_char,
         _: *const libc::c_char,
     ) -> *const RsvgNode;
-    fn rsvg_new_filter_primitive_image(
-        _: *const libc::c_char,
-        _: *const RsvgNode,
-        _: *const libc::c_char,
-        _: *const libc::c_char,
-    ) -> *const RsvgNode;
     fn rsvg_new_filter_primitive_erode(
         _: *const libc::c_char,
         _: *const RsvgNode,
@@ -129,20 +103,12 @@ lazy_static! {
     #[cfg_attr(rustfmt, rustfmt_skip)]
     static ref NODE_CREATORS_C: HashMap<&'static str, (bool, NodeCreateCFn)> = {
         let mut h = HashMap::new();
-        h.insert("feBlend",             (true,  rsvg_new_filter_primitive_blend as NodeCreateCFn));
         h.insert("feColorMatrix",       (true,  rsvg_new_filter_primitive_color_matrix as NodeCreateCFn));
-        h.insert("feComponentTransfer", (true,  rsvg_new_filter_primitive_component_transfer as NodeCreateCFn));
         h.insert("feConvolveMatrix",    (true,  rsvg_new_filter_primitive_convolve_matrix as NodeCreateCFn));
         h.insert("feDiffuseLighting",   (true,  rsvg_new_filter_primitive_diffuse_lighting as NodeCreateCFn));
         h.insert("feDisplacementMap",   (true,  rsvg_new_filter_primitive_displacement_map as NodeCreateCFn));
         h.insert("feDistantLight",      (false, rsvg_new_node_light_source as NodeCreateCFn));
-        h.insert("feFlood",             (true,  rsvg_new_filter_primitive_flood as NodeCreateCFn));
-        h.insert("feFuncA",             (false, rsvg_new_node_component_transfer_function as NodeCreateCFn));
-        h.insert("feFuncB",             (false, rsvg_new_node_component_transfer_function as NodeCreateCFn));
-        h.insert("feFuncG",             (false, rsvg_new_node_component_transfer_function as NodeCreateCFn));
-        h.insert("feFuncR",             (false, rsvg_new_node_component_transfer_function as NodeCreateCFn));
         h.insert("feGaussianBlur",      (true,  rsvg_new_filter_primitive_gaussian_blur as NodeCreateCFn));
-        h.insert("feImage",             (true,  rsvg_new_filter_primitive_image as NodeCreateCFn));
         h.insert("feMorphology",        (true,  rsvg_new_filter_primitive_erode as NodeCreateCFn));
         h.insert("fePointLight",        (false, rsvg_new_node_light_source as NodeCreateCFn));
         h.insert("feSpecularLighting",  (true,  rsvg_new_filter_primitive_specular_lighting as NodeCreateCFn));
@@ -167,12 +133,40 @@ macro_rules! node_create_fn {
 
 node_create_fn!(create_circle, Circle, NodeCircle::new);
 node_create_fn!(create_clip_path, ClipPath, NodeClipPath::new);
+node_create_fn!(create_blend, FilterPrimitiveBlend, Blend::new);
+node_create_fn!(
+    create_component_transfer,
+    FilterPrimitiveComponentTransfer,
+    ComponentTransfer::new
+);
+node_create_fn!(
+    create_component_transfer_func_r,
+    ComponentTransferFunction,
+    FuncX::new_r
+);
+node_create_fn!(
+    create_component_transfer_func_g,
+    ComponentTransferFunction,
+    FuncX::new_g
+);
+node_create_fn!(
+    create_component_transfer_func_b,
+    ComponentTransferFunction,
+    FuncX::new_b
+);
+node_create_fn!(
+    create_component_transfer_func_a,
+    ComponentTransferFunction,
+    FuncX::new_a
+);
 node_create_fn!(create_composite, FilterPrimitiveComposite, Composite::new);
 node_create_fn!(create_defs, Defs, NodeDefs::new);
 node_create_fn!(create_ellipse, Ellipse, NodeEllipse::new);
 node_create_fn!(create_filter, Filter, NodeFilter::new);
+node_create_fn!(create_flood, FilterPrimitiveFlood, Flood::new);
 node_create_fn!(create_group, Group, NodeGroup::new);
 node_create_fn!(create_image, Image, NodeImage::new);
+node_create_fn!(create_fe_image, FilterPrimitiveImage, Image::new);
 node_create_fn!(
     create_linear_gradient,
     LinearGradient,
@@ -227,7 +221,15 @@ lazy_static! {
         h.insert("defs",                (true,  create_defs as NodeCreateFn));
         /* h.insert("desc",             (true,  as NodeCreateFn)); */
         h.insert("ellipse",             (true,  create_ellipse as NodeCreateFn));
+        h.insert("feBlend",             (true,  create_blend as NodeCreateFn));
+        h.insert("feComponentTransfer", (true,  create_component_transfer as NodeCreateFn));
         h.insert("feComposite",         (true,  create_composite as NodeCreateFn));
+        h.insert("feFuncR",             (false, create_component_transfer_func_r as NodeCreateFn));
+        h.insert("feFuncG",             (false, create_component_transfer_func_g as NodeCreateFn));
+        h.insert("feFuncB",             (false, create_component_transfer_func_b as NodeCreateFn));
+        h.insert("feFuncA",             (false, create_component_transfer_func_a as NodeCreateFn));
+        h.insert("feFlood",             (true,  create_flood as NodeCreateFn));
+        h.insert("feImage",             (true,  create_fe_image as NodeCreateFn));
         h.insert("feMerge",             (true,  create_merge as NodeCreateFn));
         h.insert("feMergeNode",         (false, create_merge_node as NodeCreateFn));
         h.insert("feOffset",            (true,  create_offset as NodeCreateFn));
