@@ -2,12 +2,12 @@ use std::cell::Cell;
 use std::f64::consts::*;
 
 use cairo::MatrixTrait;
+use cssparser::{CowRcStr, Parser, Token};
 
 use aspect_ratio::*;
 use attributes::Attribute;
 use draw::add_clipping_rect;
-use drawing_ctx;
-use drawing_ctx::RsvgDrawingCtx;
+use drawing_ctx::DrawingCtx;
 use error::*;
 use float_eq_cairo::ApproxEqCairo;
 use handle::RsvgHandle;
@@ -16,7 +16,7 @@ use length::{LengthDir, RsvgLength};
 use node::*;
 use parsers;
 use parsers::ParseError;
-use parsers::{parse, Parse};
+use parsers::{parse, parse_and_validate, Parse};
 use path_builder::*;
 use property_bag::PropertyBag;
 use state::{ComputedValues, SpecifiedValue, State};
@@ -39,14 +39,25 @@ impl Parse for MarkerUnits {
     type Data = ();
     type Err = AttributeError;
 
-    fn parse(s: &str, _: ()) -> Result<MarkerUnits, AttributeError> {
-        match s {
-            "userSpaceOnUse" => Ok(MarkerUnits::UserSpaceOnUse),
-            "strokeWidth" => Ok(MarkerUnits::StrokeWidth),
-            _ => Err(AttributeError::Parse(ParseError::new(
-                "expected \"userSpaceOnUse\" or \"strokeWidth\"",
-            ))),
-        }
+    fn parse(parser: &mut Parser, _: ()) -> Result<MarkerUnits, AttributeError> {
+        let loc = parser.current_source_location();
+
+        parser
+            .expect_ident()
+            .and_then(|cow| match cow.as_ref() {
+                "userSpaceOnUse" => Ok(MarkerUnits::UserSpaceOnUse),
+                "strokeWidth" => Ok(MarkerUnits::StrokeWidth),
+                _ => Err(
+                    loc.new_basic_unexpected_token_error(Token::Ident(CowRcStr::from(
+                        cow.as_ref().to_string(),
+                    ))),
+                ),
+            })
+            .map_err(|_| {
+                AttributeError::Parse(ParseError::new(
+                    "expected \"userSpaceOnUse\" or \"strokeWidth\"",
+                ))
+            })
     }
 }
 
@@ -67,12 +78,13 @@ impl Parse for MarkerOrient {
     type Data = ();
     type Err = AttributeError;
 
-    fn parse(s: &str, _: ()) -> Result<MarkerOrient, AttributeError> {
-        match s {
-            "auto" => Ok(MarkerOrient::Auto),
-            _ => parsers::angle_degrees(s)
+    fn parse(parser: &mut Parser, _: ()) -> Result<MarkerOrient, AttributeError> {
+        if parser.try(|p| p.expect_ident_matching("auto")).is_ok() {
+            Ok(MarkerOrient::Auto)
+        } else {
+            parsers::angle_degrees(parser)
                 .map(MarkerOrient::Degrees)
-                .map_err(AttributeError::Parse),
+                .map_err(AttributeError::Parse)
         }
     }
 }
@@ -104,13 +116,13 @@ impl NodeMarker {
 
     fn get_default_size(dir: LengthDir) -> RsvgLength {
         // per the spec
-        RsvgLength::parse("3", dir).unwrap()
+        RsvgLength::parse_str("3", dir).unwrap()
     }
 
     fn render(
         &self,
         node: &RsvgNode,
-        draw_ctx: *mut RsvgDrawingCtx,
+        draw_ctx: &mut DrawingCtx,
         xpos: f64,
         ypos: f64,
         computed_angle: f64,
@@ -129,7 +141,7 @@ impl NodeMarker {
             return;
         }
 
-        let cr = drawing_ctx::get_cairo_context(draw_ctx);
+        let cr = draw_ctx.get_cairo_context();
         cr.save();
 
         let mut affine = cr.get_matrix();
@@ -159,24 +171,17 @@ impl NodeMarker {
 
             affine.scale(w / vbox.0.width, h / vbox.0.height);
 
-            cr.set_matrix(affine);
-
-            drawing_ctx::push_view_box(draw_ctx, vbox.0.width, vbox.0.height);
+            draw_ctx.push_view_box(vbox.0.width, vbox.0.height);
         } else {
-            cr.set_matrix(affine);
-
-            drawing_ctx::push_view_box(draw_ctx, marker_width, marker_height);
+            draw_ctx.push_view_box(marker_width, marker_height);
         }
 
-        drawing_ctx::push_discrete_layer(draw_ctx, &values, clipping);
-
         affine.translate(
             -self.ref_x.get().normalize(&values, draw_ctx),
             -self.ref_y.get().normalize(&values, draw_ctx),
         );
 
-        let cr2 = drawing_ctx::get_cairo_context(draw_ctx);
-        cr2.set_matrix(affine);
+        cr.set_matrix(affine);
 
         if !values.is_overflow() {
             if let Some(vbox) = self.vbox.get() {
@@ -186,11 +191,11 @@ impl NodeMarker {
             }
         }
 
-        node.draw_children(node, &cascaded, draw_ctx, false, clipping);
+        draw_ctx.with_discrete_layer(node, values, clipping, &mut |dc| {
+            node.draw_children(&cascaded, dc, clipping);
+        });
 
-        drawing_ctx::pop_discrete_layer(draw_ctx, node, &values, clipping);
-
-        drawing_ctx::pop_view_box(draw_ctx);
+        draw_ctx.pop_view_box();
 
         cr.restore();
     }
@@ -204,39 +209,33 @@ impl NodeTrait for NodeMarker {
 
         for (_key, attr, value) in pbag.iter() {
             match attr {
-                Attribute::MarkerUnits => self.units.set(parse("markerUnits", value, (), None)?),
+                Attribute::MarkerUnits => self.units.set(parse("markerUnits", value, ())?),
 
-                Attribute::RefX => {
-                    self.ref_x
-                        .set(parse("refX", value, LengthDir::Horizontal, None)?)
-                }
+                Attribute::RefX => self.ref_x.set(parse("refX", value, LengthDir::Horizontal)?),
 
-                Attribute::RefY => self
-                    .ref_y
-                    .set(parse("refY", value, LengthDir::Vertical, None)?),
+                Attribute::RefY => self.ref_y.set(parse("refY", value, LengthDir::Vertical)?),
 
-                Attribute::MarkerWidth => self.width.set(parse(
+                Attribute::MarkerWidth => self.width.set(parse_and_validate(
                     "markerWidth",
                     value,
                     LengthDir::Horizontal,
-                    Some(RsvgLength::check_nonnegative),
+                    RsvgLength::check_nonnegative,
                 )?),
 
-                Attribute::MarkerHeight => self.height.set(parse(
+                Attribute::MarkerHeight => self.height.set(parse_and_validate(
                     "markerHeight",
                     value,
                     LengthDir::Vertical,
-                    Some(RsvgLength::check_nonnegative),
+                    RsvgLength::check_nonnegative,
                 )?),
 
-                Attribute::Orient => self.orient.set(parse("orient", value, (), None)?),
+                Attribute::Orient => self.orient.set(parse("orient", value, ())?),
 
                 Attribute::PreserveAspectRatio => {
-                    self.aspect
-                        .set(parse("preserveAspectRatio", value, (), None)?)
+                    self.aspect.set(parse("preserveAspectRatio", value, ())?)
                 }
 
-                Attribute::ViewBox => self.vbox.set(Some(parse("viewBox", value, (), None)?)),
+                Attribute::ViewBox => self.vbox.set(Some(parse("viewBox", value, ())?)),
 
                 _ => (),
             }
@@ -245,7 +244,7 @@ impl NodeTrait for NodeMarker {
         Ok(())
     }
 
-    fn set_overriden_properties(&self, state: &mut State) {
+    fn set_overridden_properties(&self, state: &mut State) {
         // markers are always displayed, even if <marker> or its ancestors are display:none
         state.values.display = SpecifiedValue::Specified(Default::default());
     }
@@ -584,7 +583,7 @@ enum MarkerType {
 }
 
 fn emit_marker_by_name(
-    draw_ctx: *mut RsvgDrawingCtx,
+    draw_ctx: &mut DrawingCtx,
     name: &str,
     xpos: f64,
     ypos: f64,
@@ -592,8 +591,7 @@ fn emit_marker_by_name(
     line_width: f64,
     clipping: bool,
 ) {
-    if let Some(acquired) = drawing_ctx::get_acquired_node_of_type(draw_ctx, name, NodeType::Marker)
-    {
+    if let Some(acquired) = draw_ctx.get_acquired_node_of_type(Some(name), NodeType::Marker) {
         let node = acquired.get();
 
         node.with_impl(|marker: &NodeMarker| {
@@ -639,7 +637,7 @@ fn emit_marker<E>(
 
 pub fn render_markers_for_path_builder(
     builder: &PathBuilder,
-    draw_ctx: *mut RsvgDrawingCtx,
+    draw_ctx: &mut DrawingCtx,
     values: &ComputedValues,
     clipping: bool,
 ) {
@@ -819,21 +817,21 @@ mod parser_tests {
     #[test]
     fn parsing_invalid_marker_units_yields_error() {
         assert!(is_parse_error(
-            &MarkerUnits::parse("", ()).map_err(|e| AttributeError::from(e))
+            &MarkerUnits::parse_str("", ()).map_err(|e| AttributeError::from(e))
         ));
         assert!(is_parse_error(
-            &MarkerUnits::parse("foo", ()).map_err(|e| AttributeError::from(e))
+            &MarkerUnits::parse_str("foo", ()).map_err(|e| AttributeError::from(e))
         ));
     }
 
     #[test]
     fn parses_marker_units() {
         assert_eq!(
-            MarkerUnits::parse("userSpaceOnUse", ()),
+            MarkerUnits::parse_str("userSpaceOnUse", ()),
             Ok(MarkerUnits::UserSpaceOnUse)
         );
         assert_eq!(
-            MarkerUnits::parse("strokeWidth", ()),
+            MarkerUnits::parse_str("strokeWidth", ()),
             Ok(MarkerUnits::StrokeWidth)
         );
     }
@@ -841,35 +839,38 @@ mod parser_tests {
     #[test]
     fn parsing_invalid_marker_orient_yields_error() {
         assert!(is_parse_error(
-            &MarkerOrient::parse("", ()).map_err(|e| AttributeError::from(e))
+            &MarkerOrient::parse_str("", ()).map_err(|e| AttributeError::from(e))
         ));
         assert!(is_parse_error(
-            &MarkerOrient::parse("blah", ()).map_err(|e| AttributeError::from(e))
+            &MarkerOrient::parse_str("blah", ()).map_err(|e| AttributeError::from(e))
         ));
         assert!(is_parse_error(
-            &MarkerOrient::parse("45blah", ()).map_err(|e| AttributeError::from(e))
+            &MarkerOrient::parse_str("45blah", ()).map_err(|e| AttributeError::from(e))
         ));
     }
 
     #[test]
     fn parses_marker_orient() {
-        assert_eq!(MarkerOrient::parse("auto", ()), Ok(MarkerOrient::Auto));
+        assert_eq!(MarkerOrient::parse_str("auto", ()), Ok(MarkerOrient::Auto));
 
-        assert_eq!(MarkerOrient::parse("0", ()), Ok(MarkerOrient::Degrees(0.0)));
         assert_eq!(
-            MarkerOrient::parse("180", ()),
+            MarkerOrient::parse_str("0", ()),
+            Ok(MarkerOrient::Degrees(0.0))
+        );
+        assert_eq!(
+            MarkerOrient::parse_str("180", ()),
             Ok(MarkerOrient::Degrees(180.0))
         );
         assert_eq!(
-            MarkerOrient::parse("180deg", ()),
+            MarkerOrient::parse_str("180deg", ()),
             Ok(MarkerOrient::Degrees(180.0))
         );
         assert_eq!(
-            MarkerOrient::parse("-400grad", ()),
+            MarkerOrient::parse_str("-400grad", ()),
             Ok(MarkerOrient::Degrees(-360.0))
         );
         assert_eq!(
-            MarkerOrient::parse("1rad", ()),
+            MarkerOrient::parse_str("1rad", ()),
             Ok(MarkerOrient::Degrees(180.0 / PI))
         );
     }
