@@ -1,5 +1,6 @@
 use cairo::{Matrix, MatrixTrait};
 use downcast_rs::*;
+use glib;
 use glib::translate::*;
 use glib_sys;
 use libc;
@@ -10,14 +11,23 @@ use std::rc::{Rc, Weak};
 use std::str::FromStr;
 
 use attributes::Attribute;
+use cond::{RequiredExtensions, RequiredFeatures, SystemLanguage};
 use defs::{self, RsvgDefs};
-use drawing_ctx;
-use drawing_ctx::RsvgDrawingCtx;
+use drawing_ctx::DrawingCtx;
 use error::*;
 use handle::RsvgHandle;
 use parsers::{Parse, ParseError};
 use property_bag::PropertyBag;
-use state::{self, rsvg_state_new, ComputedValues, Overflow, RsvgState, SpecifiedValue, State};
+use state::{
+    self,
+    rsvg_state_new,
+    ComputedValues,
+    Overflow,
+    RsvgState,
+    SpecifiedValue,
+    SpecifiedValues,
+    State,
+};
 use util::utf8_cstr;
 
 // A *const RsvgNode is just a pointer for the C code's benefit: it
@@ -82,9 +92,7 @@ impl<'a> CascadedValues<'a> {
     /// `<use>`'s own cascade, not wih the element's original cascade.
     pub fn new_from_values(node: &'a Node, values: &ComputedValues) -> CascadedValues<'a> {
         let mut v = values.clone();
-        let state = node.get_state();
-
-        state.to_computed_values(&mut v);
+        node.get_specified_values().to_computed_values(&mut v);
 
         CascadedValues {
             inner: CascadedInner::FromValues(v),
@@ -117,14 +125,13 @@ pub trait NodeTrait: Downcast {
 
     /// Sets any special-cased properties that the node may have, that are different
     /// from defaults in the node's `State`.
-    fn set_overriden_properties(&self, _state: &mut State) {}
+    fn set_overridden_properties(&self, _state: &mut State) {}
 
     fn draw(
         &self,
         _node: &RsvgNode,
         _cascaded: &CascadedValues,
-        _draw_ctx: *mut RsvgDrawingCtx,
-        _with_layer: bool,
+        _draw_ctx: &mut DrawingCtx,
         _clipping: bool,
     ) {
         // by default nodes don't draw themselves
@@ -172,10 +179,12 @@ pub struct Node {
     result: RefCell<NodeResult>,
     transform: Cell<Matrix>,
     values: RefCell<ComputedValues>,
+    cond: Cell<bool>,
     node_impl: Box<NodeTrait>,
 }
 
 // An iterator over the Node's children
+#[derive(Clone)]
 pub struct Children {
     next: Option<Rc<Node>>,
     next_back: Option<Rc<Node>>,
@@ -261,6 +270,7 @@ impl Node {
             transform: Cell::new(Matrix::identity()),
             result: RefCell::new(Ok(())),
             values: RefCell::new(ComputedValues::default()),
+            cond: Cell::new(true),
             node_impl,
         }
     }
@@ -287,16 +297,12 @@ impl Node {
         self.transform.get()
     }
 
-    pub fn get_state(&self) -> &State {
-        state::from_c(self.state)
-    }
-
     pub fn get_state_mut(&self) -> &mut State {
         state::from_c_mut(self.state)
     }
 
-    pub fn set_computed_values(&self, values: &ComputedValues) {
-        *self.values.borrow_mut() = values.clone();
+    pub fn get_specified_values(&self) -> &SpecifiedValues {
+        state::from_c(self.state).get_specified_values()
     }
 
     pub fn get_cascaded_values(&self) -> CascadedValues {
@@ -305,16 +311,18 @@ impl Node {
 
     pub fn cascade(&self, values: &ComputedValues) {
         let mut values = values.clone();
-        let state = self.get_state();
-
-        state.to_computed_values(&mut values);
-        self.set_computed_values(&values);
+        self.get_specified_values().to_computed_values(&mut values);
+        *self.values.borrow_mut() = values.clone();
 
         for child in self.children() {
             child.cascade(&values);
         }
     }
 
+    pub fn get_cond(&self) -> bool {
+        self.cond.get()
+    }
+
     pub fn get_parent(&self) -> Option<Rc<Node>> {
         match self.parent {
             None => None,
@@ -353,9 +361,8 @@ impl Node {
     pub fn set_atts(&self, node: &RsvgNode, handle: *const RsvgHandle, pbag: &PropertyBag) {
         for (_key, attr, value) in pbag.iter() {
             match attr {
-                Attribute::Transform => match Matrix::parse(value, ()) {
+                Attribute::Transform => match Matrix::parse_str(value, ()) {
                     Ok(affine) => self.transform.set(affine),
-
                     Err(e) => {
                         self.set_error(NodeError::attribute_error(Attribute::Transform, e));
                         return;
@@ -366,30 +373,72 @@ impl Node {
             }
         }
 
+        match self.parse_conditional_processing_attributes(pbag) {
+            Ok(_) => (),
+            Err(e) => {
+                self.set_error(e);
+                return;
+            }
+        }
+
         *self.result.borrow_mut() = self.node_impl.set_atts(node, handle, pbag);
     }
 
-    pub fn set_overriden_properties(&self) {
+    fn parse_conditional_processing_attributes(&self, pbag: &PropertyBag) -> Result<(), NodeError> {
+        let mut cond = self.cond.get();
+
+        for (_key, attr, value) in pbag.iter() {
+            // FIXME: move this to "do catch" when we can bump the rustc version dependency
+            let mut parse = || {
+                match attr {
+                    Attribute::RequiredExtensions if cond => {
+                        cond = RequiredExtensions::from_attribute(value)
+                            .map(|RequiredExtensions(res)| res)?;
+                    }
+
+                    Attribute::RequiredFeatures if cond => {
+                        cond = RequiredFeatures::from_attribute(value)
+                            .map(|RequiredFeatures(res)| res)?;
+                    }
+
+                    Attribute::SystemLanguage if cond => {
+                        cond = SystemLanguage::from_attribute(value, &glib::get_language_names())
+                            .map(|SystemLanguage(res, _)| res)?;
+                    }
+
+                    _ => {}
+                }
+
+                Ok(cond)
+            };
+
+            parse()
+                .map(|c| self.cond.set(c))
+                .map_err(|e| NodeError::attribute_error(attr, e))?;
+        }
+
+        Ok(())
+    }
+
+    pub fn set_overridden_properties(&self) {
         let mut state = self.get_state_mut();
-        self.node_impl.set_overriden_properties(&mut state);
+        self.node_impl.set_overridden_properties(&mut state);
     }
 
     pub fn draw(
         &self,
         node: &RsvgNode,
         cascaded: &CascadedValues,
-        draw_ctx: *mut RsvgDrawingCtx,
-        with_layer: bool,
+        draw_ctx: &mut DrawingCtx,
         clipping: bool,
     ) {
         if self.result.borrow().is_ok() {
-            let cr = drawing_ctx::get_cairo_context(draw_ctx);
+            let cr = draw_ctx.get_cairo_context();
             let save_affine = cr.get_matrix();
 
             cr.transform(self.get_transform());
 
-            self.node_impl
-                .draw(node, cascaded, draw_ctx, with_layer, clipping);
+            self.node_impl.draw(node, cascaded, draw_ctx, clipping);
 
             cr.set_matrix(save_affine);
         }
@@ -429,30 +478,12 @@ impl Node {
 
     pub fn draw_children(
         &self,
-        node: &RsvgNode,
         cascaded: &CascadedValues,
-        draw_ctx: *mut RsvgDrawingCtx,
-        with_layer: bool,
+        draw_ctx: &mut DrawingCtx,
         clipping: bool,
     ) {
-        let values = cascaded.get();
-
-        if with_layer {
-            drawing_ctx::push_discrete_layer(draw_ctx, values, clipping);
-        }
-
         for child in self.children() {
-            drawing_ctx::draw_node_from_stack(
-                draw_ctx,
-                &CascadedValues::new(cascaded, &child),
-                &child,
-                true,
-                clipping,
-            );
-        }
-
-        if with_layer {
-            drawing_ctx::pop_discrete_layer(draw_ctx, node, values, clipping);
+            draw_ctx.draw_node_from_stack(&CascadedValues::new(cascaded, &child), &child, clipping);
         }
     }
 
@@ -613,7 +644,7 @@ pub extern "C" fn rsvg_node_unref(raw_node: *mut RsvgNode) -> *mut RsvgNode {
 // See https://github.com/rust-lang/rust/issues/36497 - this is what
 // added Rc::ptr_eq(), but we don't want to depend on unstable Rust
 // just yet.
-fn rc_node_ptr_eq<T: ?Sized>(this: &Rc<T>, other: &Rc<T>) -> bool {
+pub fn rc_node_ptr_eq<T: ?Sized>(this: &Rc<T>, other: &Rc<T>) -> bool {
     let this_ptr: *const T = &**this;
     let other_ptr: *const T = &**other;
     this_ptr == other_ptr
@@ -674,12 +705,12 @@ pub extern "C" fn rsvg_node_set_atts(
 }
 
 #[no_mangle]
-pub extern "C" fn rsvg_node_set_overriden_properties(raw_node: *mut RsvgNode) {
+pub extern "C" fn rsvg_node_set_overridden_properties(raw_node: *mut RsvgNode) {
     assert!(!raw_node.is_null());
 
     let node: &RsvgNode = unsafe { &*raw_node };
 
-    node.set_overriden_properties();
+    node.set_overridden_properties();
 }
 
 #[no_mangle]
