@@ -36,7 +36,60 @@ pub trait Parse: Sized {
     type Data;
     type Err;
 
-    fn parse(s: &str, data: Self::Data) -> Result<Self, Self::Err>;
+    fn parse(parser: &mut Parser, data: Self::Data) -> Result<Self, Self::Err>;
+
+    fn parse_str(s: &str, data: Self::Data) -> Result<Self, Self::Err> {
+        let mut input = ParserInput::new(s);
+        let mut parser = Parser::new(&mut input);
+
+        Self::parse(&mut parser, data).and_then(|r| {
+            // FIXME: parser.expect_exhausted()?;
+            Ok(r)
+        })
+    }
+}
+
+impl Parse for f64 {
+    type Data = ();
+    type Err = AttributeError;
+
+    fn parse(parser: &mut Parser, _: ()) -> Result<f64, AttributeError> {
+        Ok(f64::from(parser.expect_number().map_err(|_| {
+            AttributeError::Parse(ParseError::new("expected number"))
+        })?))
+    }
+}
+
+impl Parse for String {
+    type Data = ();
+    type Err = AttributeError;
+
+    fn parse(parser: &mut Parser, _: ()) -> Result<String, AttributeError> {
+        Ok(String::from(
+            parser
+                .expect_string()
+                .map_err(|_| AttributeError::Parse(ParseError::new("expected number")))?
+                .as_ref(),
+        ))
+    }
+}
+
+/// Parses a `value` string into a type `T`.
+///
+/// Some value types need some extra `data` to be parsed.  This
+/// corresponds to the `<T as Parse>::Data` associated type.  For
+/// example, an `RsvgLength` has an associated `type Data =
+/// LengthDir`, so to parse a length value, you could specify
+/// `LengthDir::Horizontal` for `data`, for example.
+pub fn parse<T>(key: &str, value: &str, data: <T as Parse>::Data) -> Result<T, NodeError>
+where
+    T: Parse<Err = AttributeError>,
+{
+    let mut input = ParserInput::new(value);
+    let mut parser = Parser::new(&mut input);
+
+    T::parse(&mut parser, data)
+        .map_err(|e| NodeError::attribute_error(Attribute::from_str(key).unwrap(), e))
 }
 
 /// Parses a `value` string into a type `T` with an optional validation function.
@@ -46,23 +99,21 @@ pub trait Parse: Sized {
 /// example, an `RsvgLength` has an associated `type Data =
 /// LengthDir`, so to parse a length value, you could specify
 /// `LengthDir::Horizontal` for `data`, for example.
-pub fn parse<T>(
+pub fn parse_and_validate<T, F>(
     key: &str,
     value: &str,
     data: <T as Parse>::Data,
-    validate: Option<fn(T) -> Result<T, AttributeError>>,
+    validate: F,
 ) -> Result<T, NodeError>
 where
     T: Parse<Err = AttributeError>,
+    F: FnOnce(T) -> Result<T, AttributeError>,
 {
-    T::parse(value, data)
-        .and_then(|v| {
-            if let Some(validate) = validate {
-                validate(v)
-            } else {
-                Ok(v)
-            }
-        })
+    let mut input = ParserInput::new(value);
+    let mut parser = Parser::new(&mut input);
+
+    T::parse(&mut parser, data)
+        .and_then(validate)
         .map_err(|e| NodeError::attribute_error(Attribute::from_str(key).unwrap(), e))
 }
 
@@ -73,10 +124,7 @@ where
 //
 // Returns an f64 angle in degrees
 
-pub fn angle_degrees(s: &str) -> Result<f64, ParseError> {
-    let mut input = ParserInput::new(s);
-    let mut parser = Parser::new(&mut input);
-
+pub fn angle_degrees(parser: &mut Parser) -> Result<f64, ParseError> {
     let angle = {
         let token = parser
             .next()
@@ -215,9 +263,11 @@ pub fn list_of_points(string: &str) -> Result<Vec<(f64, f64)>, ParseError> {
 
 // Lists of number values
 
+#[derive(Eq, PartialEq)]
 pub enum ListLength {
     Exact(usize),
     Maximum(usize),
+    Unbounded,
 }
 
 #[derive(Debug, PartialEq)]
@@ -226,37 +276,53 @@ pub enum NumberListError {
     Parse(ParseError),
 }
 
-pub fn number_list(s: &str, length: ListLength) -> Result<Vec<f64>, NumberListError> {
+pub fn number_list(parser: &mut Parser, length: ListLength) -> Result<Vec<f64>, NumberListError> {
     let n;
 
     match length {
         ListLength::Exact(l) => {
             assert!(l > 0);
-            n = l;
+            n = Some(l);
         }
         ListLength::Maximum(l) => {
             assert!(l > 0);
-            n = l;
+            n = Some(l);
+        }
+        ListLength::Unbounded => {
+            n = None;
         }
     }
 
-    let mut input = ParserInput::new(s);
-    let mut parser = Parser::new(&mut input);
+    let mut v = Vec::<f64>::with_capacity(n.unwrap_or(0));
 
-    let mut v = Vec::<f64>::with_capacity(n);
+    if parser.is_exhausted() && length == ListLength::Unbounded {
+        return Ok(v);
+    }
+
+    for i in 0.. {
+        if i != 0 {
+            optional_comma(parser);
+        }
 
-    for i in 0..n {
         v.push(f64::from(parser.expect_number().map_err(|_| {
             NumberListError::Parse(ParseError::new("expected number"))
         })?));
 
-        if i != n - 1 {
-            optional_comma(&mut parser);
+        match length {
+            ListLength::Exact(l) | ListLength::Maximum(l) => if i + 1 == l {
+                break;
+            },
+            _ => (),
         }
 
         if parser.is_exhausted() {
-            if let ListLength::Maximum(_) = length {
-                break;
+            match length {
+                ListLength::Exact(l) => {
+                    if i + 1 == l {
+                        break;
+                    }
+                }
+                _ => break,
             }
         }
     }
@@ -268,6 +334,13 @@ pub fn number_list(s: &str, length: ListLength) -> Result<Vec<f64>, NumberListEr
     Ok(v)
 }
 
+pub fn number_list_from_str(s: &str, length: ListLength) -> Result<Vec<f64>, NumberListError> {
+    let mut input = ParserInput::new(s);
+    let mut parser = Parser::new(&mut input);
+
+    number_list(&mut parser, length)
+}
+
 #[repr(C)]
 #[derive(Debug, Copy, Clone, PartialEq)]
 pub enum NumberListLength {
@@ -294,7 +367,7 @@ pub extern "C" fn rsvg_css_parse_number_list(
 
     let s = unsafe { utf8_cstr(in_str) };
 
-    let result = number_list(s, length);
+    let result = number_list_from_str(s, length);
 
     match result {
         Ok(number_list) => {
@@ -375,62 +448,84 @@ mod tests {
         assert!(list_of_points("1 2-3,-4").is_err());
     }
 
+    fn angle_degrees_str(s: &str) -> Result<f64, ParseError> {
+        let mut input = ParserInput::new(s);
+        let mut parser = Parser::new(&mut input);
+
+        angle_degrees(&mut parser)
+    }
+
     #[test]
     fn parses_angle() {
-        assert_eq!(angle_degrees("0"), Ok(0.0));
-        assert_eq!(angle_degrees("15"), Ok(15.0));
-        assert_eq!(angle_degrees("180.5deg"), Ok(180.5));
-        assert_eq!(angle_degrees("1rad"), Ok(180.0 / PI));
-        assert_eq!(angle_degrees("-400grad"), Ok(-360.0));
+        assert_eq!(angle_degrees_str("0"), Ok(0.0));
+        assert_eq!(angle_degrees_str("15"), Ok(15.0));
+        assert_eq!(angle_degrees_str("180.5deg"), Ok(180.5));
+        assert_eq!(angle_degrees_str("1rad"), Ok(180.0 / PI));
+        assert_eq!(angle_degrees_str("-400grad"), Ok(-360.0));
 
-        assert!(angle_degrees("").is_err());
-        assert!(angle_degrees("foo").is_err());
-        assert!(angle_degrees("300foo").is_err());
+        assert!(angle_degrees_str("").is_err());
+        assert!(angle_degrees_str("foo").is_err());
+        assert!(angle_degrees_str("300foo").is_err());
     }
 
     #[test]
     fn parses_number_list() {
-        assert_eq!(number_list("5", ListLength::Exact(1)), Ok(vec![5.0]));
+        assert_eq!(
+            number_list_from_str("5", ListLength::Exact(1)),
+            Ok(vec![5.0])
+        );
 
         assert_eq!(
-            number_list("1 2 3 4", ListLength::Exact(4)),
+            number_list_from_str("1 2 3 4", ListLength::Exact(4)),
             Ok(vec![1.0, 2.0, 3.0, 4.0])
         );
 
-        assert_eq!(number_list("5", ListLength::Maximum(1)), Ok(vec![5.0]));
+        assert_eq!(
+            number_list_from_str("5", ListLength::Maximum(1)),
+            Ok(vec![5.0])
+        );
 
         assert_eq!(
-            number_list("1.0, -2.5", ListLength::Maximum(2)),
+            number_list_from_str("1.0, -2.5", ListLength::Maximum(2)),
             Ok(vec![1.0, -2.5])
         );
 
         assert_eq!(
-            number_list("5 6", ListLength::Maximum(3)),
+            number_list_from_str("5 6", ListLength::Maximum(3)),
             Ok(vec![5.0, 6.0])
         );
+
+        assert_eq!(number_list_from_str("", ListLength::Unbounded), Ok(vec![]));
+        assert_eq!(
+            number_list_from_str("1, 2, 3.0, 4, 5", ListLength::Unbounded),
+            Ok(vec![1.0, 2.0, 3.0, 4.0, 5.0])
+        );
     }
 
     #[test]
     fn errors_on_invalid_number_list() {
         // empty
-        assert!(number_list("", ListLength::Exact(1)).is_err());
+        assert!(number_list_from_str("", ListLength::Exact(1)).is_err());
 
         // garbage
-        assert!(number_list("foo", ListLength::Exact(1)).is_err());
-        assert!(number_list("1foo", ListLength::Exact(2)).is_err());
-        assert!(number_list("1 foo", ListLength::Exact(2)).is_err());
-        assert!(number_list("1 foo 2", ListLength::Exact(2)).is_err());
-        assert!(number_list("1,foo", ListLength::Exact(2)).is_err());
+        assert!(number_list_from_str("foo", ListLength::Exact(1)).is_err());
+        assert!(number_list_from_str("1foo", ListLength::Exact(2)).is_err());
+        assert!(number_list_from_str("1 foo", ListLength::Exact(2)).is_err());
+        assert!(number_list_from_str("1 foo 2", ListLength::Exact(2)).is_err());
+        assert!(number_list_from_str("1,foo", ListLength::Exact(2)).is_err());
 
         // too many
-        assert!(number_list("1 2", ListLength::Exact(1)).is_err());
-        assert!(number_list("1,2,3", ListLength::Maximum(2)).is_err());
+        assert!(number_list_from_str("1 2", ListLength::Exact(1)).is_err());
+        assert!(number_list_from_str("1,2,3", ListLength::Maximum(2)).is_err());
 
         // extra token
-        assert!(number_list("1,", ListLength::Exact(1)).is_err());
+        assert!(number_list_from_str("1,", ListLength::Exact(1)).is_err());
+        assert!(number_list_from_str("1,", ListLength::Exact(1)).is_err());
+        assert!(number_list_from_str("1,", ListLength::Maximum(1)).is_err());
+        assert!(number_list_from_str("1,", ListLength::Unbounded).is_err());
 
         // too few
-        assert!(number_list("1", ListLength::Exact(2)).is_err());
-        assert!(number_list("1 2", ListLength::Exact(3)).is_err());
+        assert!(number_list_from_str("1", ListLength::Exact(2)).is_err());
+        assert!(number_list_from_str("1 2", ListLength::Exact(3)).is_err());
     }
 }
