@@ -3,13 +3,13 @@ use std::cell::RefCell;
 
 use attributes::Attribute;
 use draw::draw_path_builder;
-use drawing_ctx::{self, RsvgDrawingCtx};
+use drawing_ctx::DrawingCtx;
 use error::*;
 use handle::RsvgHandle;
 use length::*;
 use marker;
 use node::*;
-use parsers::{self, parse};
+use parsers::{self, parse, parse_and_validate};
 use path_builder::*;
 use path_parser;
 use property_bag::PropertyBag;
@@ -17,17 +17,15 @@ use state::ComputedValues;
 
 fn render_path_builder(
     builder: &PathBuilder,
-    draw_ctx: *mut RsvgDrawingCtx,
+    draw_ctx: &mut DrawingCtx,
     node: &RsvgNode,
     values: &ComputedValues,
     render_markers: bool,
     clipping: bool,
 ) {
-    drawing_ctx::push_discrete_layer(draw_ctx, values, clipping);
-
-    draw_path_builder(draw_ctx, values, builder, clipping);
-
-    drawing_ctx::pop_discrete_layer(draw_ctx, node, values, clipping);
+    draw_ctx.with_discrete_layer(node, values, clipping, &mut |dc| {
+        draw_path_builder(dc, values, builder, clipping);
+    });
 
     if render_markers {
         marker::render_markers_for_path_builder(builder, draw_ctx, values, clipping);
@@ -39,7 +37,7 @@ fn render_ellipse(
     cy: f64,
     rx: f64,
     ry: f64,
-    draw_ctx: *mut RsvgDrawingCtx,
+    draw_ctx: &mut DrawingCtx,
     node: &RsvgNode,
     values: &ComputedValues,
     clipping: bool,
@@ -132,8 +130,7 @@ impl NodeTrait for NodePath {
         &self,
         node: &RsvgNode,
         cascaded: &CascadedValues,
-        draw_ctx: *mut RsvgDrawingCtx,
-        _with_layer: bool,
+        draw_ctx: &mut DrawingCtx,
         clipping: bool,
     ) {
         let values = cascaded.get();
@@ -198,8 +195,7 @@ impl NodeTrait for NodePoly {
         &self,
         node: &RsvgNode,
         cascaded: &CascadedValues,
-        draw_ctx: *mut RsvgDrawingCtx,
-        _with_layer: bool,
+        draw_ctx: &mut DrawingCtx,
         clipping: bool,
     ) {
         let values = cascaded.get();
@@ -246,14 +242,10 @@ impl NodeTrait for NodeLine {
     fn set_atts(&self, _: &RsvgNode, _: *const RsvgHandle, pbag: &PropertyBag) -> NodeResult {
         for (_key, attr, value) in pbag.iter() {
             match attr {
-                Attribute::X1 => self
-                    .x1
-                    .set(parse("x1", value, LengthDir::Horizontal, None)?),
-                Attribute::Y1 => self.y1.set(parse("y1", value, LengthDir::Vertical, None)?),
-                Attribute::X2 => self
-                    .x2
-                    .set(parse("x2", value, LengthDir::Horizontal, None)?),
-                Attribute::Y2 => self.y2.set(parse("y2", value, LengthDir::Vertical, None)?),
+                Attribute::X1 => self.x1.set(parse("x1", value, LengthDir::Horizontal)?),
+                Attribute::Y1 => self.y1.set(parse("y1", value, LengthDir::Vertical)?),
+                Attribute::X2 => self.x2.set(parse("x2", value, LengthDir::Horizontal)?),
+                Attribute::Y2 => self.y2.set(parse("y2", value, LengthDir::Vertical)?),
                 _ => (),
             }
         }
@@ -265,8 +257,7 @@ impl NodeTrait for NodeLine {
         &self,
         node: &RsvgNode,
         cascaded: &CascadedValues,
-        draw_ctx: *mut RsvgDrawingCtx,
-        _with_layer: bool,
+        draw_ctx: &mut DrawingCtx,
         clipping: bool,
     ) {
         let values = cascaded.get();
@@ -315,35 +306,35 @@ impl NodeTrait for NodeRect {
     fn set_atts(&self, _: &RsvgNode, _: *const RsvgHandle, pbag: &PropertyBag) -> NodeResult {
         for (_key, attr, value) in pbag.iter() {
             match attr {
-                Attribute::X => self.x.set(parse("x", value, LengthDir::Horizontal, None)?),
-                Attribute::Y => self.y.set(parse("y", value, LengthDir::Vertical, None)?),
-                Attribute::Width => self.w.set(parse(
+                Attribute::X => self.x.set(parse("x", value, LengthDir::Horizontal)?),
+                Attribute::Y => self.y.set(parse("y", value, LengthDir::Vertical)?),
+                Attribute::Width => self.w.set(parse_and_validate(
                     "width",
                     value,
                     LengthDir::Horizontal,
-                    Some(RsvgLength::check_nonnegative),
+                    RsvgLength::check_nonnegative,
                 )?),
-                Attribute::Height => self.h.set(parse(
+                Attribute::Height => self.h.set(parse_and_validate(
                     "height",
                     value,
                     LengthDir::Vertical,
-                    Some(RsvgLength::check_nonnegative),
+                    RsvgLength::check_nonnegative,
                 )?),
 
                 Attribute::Rx => self.rx.set(
-                    parse(
+                    parse_and_validate(
                         "rx",
                         value,
                         LengthDir::Horizontal,
-                        Some(RsvgLength::check_nonnegative),
+                        RsvgLength::check_nonnegative,
                     ).map(Some)?,
                 ),
                 Attribute::Ry => self.ry.set(
-                    parse(
+                    parse_and_validate(
                         "ry",
                         value,
                         LengthDir::Vertical,
-                        Some(RsvgLength::check_nonnegative),
+                        RsvgLength::check_nonnegative,
                     ).map(Some)?,
                 ),
 
@@ -358,8 +349,7 @@ impl NodeTrait for NodeRect {
         &self,
         node: &RsvgNode,
         cascaded: &CascadedValues,
-        draw_ctx: *mut RsvgDrawingCtx,
-        _with_layer: bool,
+        draw_ctx: &mut DrawingCtx,
         clipping: bool,
     ) {
         let values = cascaded.get();
@@ -522,15 +512,13 @@ impl NodeTrait for NodeCircle {
     fn set_atts(&self, _: &RsvgNode, _: *const RsvgHandle, pbag: &PropertyBag) -> NodeResult {
         for (_key, attr, value) in pbag.iter() {
             match attr {
-                Attribute::Cx => self
-                    .cx
-                    .set(parse("cx", value, LengthDir::Horizontal, None)?),
-                Attribute::Cy => self.cy.set(parse("cy", value, LengthDir::Vertical, None)?),
-                Attribute::R => self.r.set(parse(
+                Attribute::Cx => self.cx.set(parse("cx", value, LengthDir::Horizontal)?),
+                Attribute::Cy => self.cy.set(parse("cy", value, LengthDir::Vertical)?),
+                Attribute::R => self.r.set(parse_and_validate(
                     "r",
                     value,
                     LengthDir::Both,
-                    Some(RsvgLength::check_nonnegative),
+                    RsvgLength::check_nonnegative,
                 )?),
 
                 _ => (),
@@ -544,8 +532,7 @@ impl NodeTrait for NodeCircle {
         &self,
         node: &RsvgNode,
         cascaded: &CascadedValues,
-        draw_ctx: *mut RsvgDrawingCtx,
-        _with_layer: bool,
+        draw_ctx: &mut DrawingCtx,
         clipping: bool,
     ) {
         let values = cascaded.get();
@@ -580,22 +567,20 @@ impl NodeTrait for NodeEllipse {
     fn set_atts(&self, _: &RsvgNode, _: *const RsvgHandle, pbag: &PropertyBag) -> NodeResult {
         for (_key, attr, value) in pbag.iter() {
             match attr {
-                Attribute::Cx => self
-                    .cx
-                    .set(parse("cx", value, LengthDir::Horizontal, None)?),
-                Attribute::Cy => self.cy.set(parse("cy", value, LengthDir::Vertical, None)?),
+                Attribute::Cx => self.cx.set(parse("cx", value, LengthDir::Horizontal)?),
+                Attribute::Cy => self.cy.set(parse("cy", value, LengthDir::Vertical)?),
 
-                Attribute::Rx => self.rx.set(parse(
+                Attribute::Rx => self.rx.set(parse_and_validate(
                     "rx",
                     value,
                     LengthDir::Horizontal,
-                    Some(RsvgLength::check_nonnegative),
+                    RsvgLength::check_nonnegative,
                 )?),
-                Attribute::Ry => self.ry.set(parse(
+                Attribute::Ry => self.ry.set(parse_and_validate(
                     "ry",
                     value,
                     LengthDir::Vertical,
-                    Some(RsvgLength::check_nonnegative),
+                    RsvgLength::check_nonnegative,
                 )?),
 
                 _ => (),
@@ -609,8 +594,7 @@ impl NodeTrait for NodeEllipse {
         &self,
         node: &RsvgNode,
         cascaded: &CascadedValues,
-        draw_ctx: *mut RsvgDrawingCtx,
-        _with_layer: bool,
+        draw_ctx: &mut DrawingCtx,
         clipping: bool,
     ) {
         let values = cascaded.get();
