@@ -1,5 +1,4 @@
-use cssparser;
-use glib;
+use cssparser::{self, Parser, Token};
 use glib::translate::*;
 use glib_sys;
 use libc;
@@ -9,14 +8,13 @@ use std::str::FromStr;
 
 use attributes::Attribute;
 use color::rgba_to_argb;
-use cond::{RequiredExtensions, RequiredFeatures, SystemLanguage};
 use error::*;
 use handle::RsvgHandle;
 use iri::IRI;
 use length::{Dasharray, LengthDir, LengthUnit, RsvgLength};
 use node::RsvgNode;
 use paint_server::PaintServer;
-use parsers::Parse;
+use parsers::{Parse, ParseError};
 use property_bag::PropertyBag;
 use property_macros::Property;
 use unitinterval::UnitInterval;
@@ -92,7 +90,6 @@ pub enum RsvgState {}
 pub struct State {
     pub values: SpecifiedValues,
     important_styles: RefCell<HashSet<Attribute>>,
-    pub cond: bool,
 }
 
 #[derive(Default, Clone)]
@@ -102,6 +99,7 @@ pub struct SpecifiedValues {
     pub clip_rule: SpecifiedValue<ClipRule>,
     pub comp_op: SpecifiedValue<CompOp>,
     pub color: SpecifiedValue<Color>,
+    pub color_interpolation_filters: SpecifiedValue<ColorInterpolationFilters>,
     pub direction: SpecifiedValue<Direction>,
     pub display: SpecifiedValue<Display>,
     pub enable_background: SpecifiedValue<EnableBackground>,
@@ -156,6 +154,7 @@ pub struct ComputedValues {
     pub clip_rule: ClipRule,
     pub comp_op: CompOp,
     pub color: Color,
+    pub color_interpolation_filters: ColorInterpolationFilters,
     pub direction: Direction,
     pub display: Display,
     pub enable_background: EnableBackground,
@@ -232,6 +231,7 @@ impl Default for ComputedValues {
             clip_path: Default::default(),
             clip_rule: Default::default(),
             color: Default::default(),
+            color_interpolation_filters: Default::default(),
             comp_op: Default::default(),
             direction: Default::default(),
             display: Default::default(),
@@ -286,12 +286,13 @@ macro_rules! compute_value {
 }
 
 impl SpecifiedValues {
-    fn to_computed_values(&self, computed: &mut ComputedValues) {
+    pub fn to_computed_values(&self, computed: &mut ComputedValues) {
         compute_value!(self, computed, baseline_shift);
         compute_value!(self, computed, clip_path);
         compute_value!(self, computed, clip_rule);
         compute_value!(self, computed, comp_op);
         compute_value!(self, computed, color);
+        compute_value!(self, computed, color_interpolation_filters);
         compute_value!(self, computed, direction);
         compute_value!(self, computed, display);
         compute_value!(self, computed, enable_background);
@@ -335,6 +336,14 @@ impl SpecifiedValues {
         compute_value!(self, computed, xml_lang);
         compute_value!(self, computed, xml_space);
     }
+
+    pub fn is_overflow(&self) -> bool {
+        match self.overflow {
+            SpecifiedValue::Specified(Overflow::Auto)
+            | SpecifiedValue::Specified(Overflow::Visible) => true,
+            _ => false,
+        }
+    }
 }
 
 impl State {
@@ -342,7 +351,6 @@ impl State {
         State {
             values: Default::default(),
             important_styles: Default::default(),
-            cond: true,
         }
     }
 
@@ -381,6 +389,10 @@ impl State {
                     self.values.color = parse_property(value, ())?;
                 }
 
+                Attribute::ColorInterpolationFilters => {
+                    self.values.color_interpolation_filters = parse_property(value, ())?;
+                }
+
                 Attribute::CompOp => {
                     self.values.comp_op = parse_property(value, ())?;
                 }
@@ -555,14 +567,16 @@ impl State {
                     // xml:lang is not a property; it is a non-presentation attribute and as such
                     // cannot have the "inherit" value.  So, we don't call parse_property() for it,
                     // but rather call its parser directly.
-                    self.values.xml_lang = SpecifiedValue::Specified(XmlLang::parse(value, ())?);
+                    self.values.xml_lang =
+                        SpecifiedValue::Specified(XmlLang::parse_str(value, ())?);
                 }
 
                 Attribute::XmlSpace => {
                     // xml:space is not a property; it is a non-presentation attribute and as such
                     // cannot have the "inherit" value.  So, we don't call parse_property() for it,
                     // but rather call its parser directly.
-                    self.values.xml_space = SpecifiedValue::Specified(XmlSpace::parse(value, ())?);
+                    self.values.xml_space =
+                        SpecifiedValue::Specified(XmlSpace::parse_str(value, ())?);
                 }
 
                 _ => {
@@ -592,41 +606,6 @@ impl State {
         Ok(())
     }
 
-    pub fn parse_conditional_processing_attributes(
-        &mut self,
-        pbag: &PropertyBag,
-    ) -> Result<(), NodeError> {
-        for (_key, attr, value) in pbag.iter() {
-            // FIXME: move this to "do catch" when we can bump the rustc version dependency
-            let mut parse = || {
-                match attr {
-                    Attribute::RequiredExtensions if self.cond => {
-                        self.cond = RequiredExtensions::parse(value, ())
-                            .map(|RequiredExtensions(res)| res)?;
-                    }
-
-                    Attribute::RequiredFeatures if self.cond => {
-                        self.cond =
-                            RequiredFeatures::parse(value, ()).map(|RequiredFeatures(res)| res)?;
-                    }
-
-                    Attribute::SystemLanguage if self.cond => {
-                        self.cond = SystemLanguage::parse(value, &glib::get_language_names())
-                            .map(|SystemLanguage(res, _)| res)?;
-                    }
-
-                    _ => {}
-                }
-
-                Ok(())
-            };
-
-            parse().map_err(|e| NodeError::attribute_error(attr, e))?;
-        }
-
-        Ok(())
-    }
-
     fn parse_style_declarations(&mut self, declarations: &str) -> Result<(), NodeError> {
         // Split an attribute value like style="foo: bar; baz: beep;" into
         // individual CSS declarations ("foo: bar" and "baz: beep") and
@@ -673,24 +652,8 @@ impl State {
         Ok(())
     }
 
-    pub fn is_overflow(&self) -> bool {
-        match self.values.overflow {
-            SpecifiedValue::Specified(Overflow::Auto)
-            | SpecifiedValue::Specified(Overflow::Visible) => true,
-            _ => false,
-        }
-    }
-
-    pub fn get_computed_values(&self) -> ComputedValues {
-        let mut computed = ComputedValues::default();
-
-        self.to_computed_values(&mut computed);
-
-        computed
-    }
-
-    pub fn to_computed_values(&self, values: &mut ComputedValues) {
-        self.values.to_computed_values(values);
+    pub fn get_specified_values(&self) -> &SpecifiedValues {
+        &self.values
     }
 }
 
@@ -708,7 +671,7 @@ where
     if value.trim() == "inherit" {
         Ok(SpecifiedValue::Inherit)
     } else {
-        Parse::parse(value, data).map(SpecifiedValue::Specified)
+        Parse::parse_str(value, data).map(SpecifiedValue::Specified)
     }
 }
 
@@ -716,7 +679,7 @@ where
 make_property!(
     ComputedValues,
     BaselineShift,
-    default: RsvgLength::parse("0.0", LengthDir::Both).unwrap(),
+    default: RsvgLength::parse_str("0.0", LengthDir::Both).unwrap(),
     newtype: RsvgLength,
     property_impl: {
         impl Property<ComputedValues> for BaselineShift {
@@ -744,13 +707,36 @@ make_property!(
 
             // These values come from Inkscape's SP_CSS_BASELINE_SHIFT_(SUB/SUPER/BASELINE);
             // see sp_style_merge_baseline_shift_from_parent()
-            fn parse(s: &str, _: Self::Data) -> Result<BaselineShift, ::error::AttributeError> {
-                match s.trim() {
-                    "baseline" => Ok(BaselineShift(RsvgLength::new(0.0, LengthUnit::Percent, LengthDir::Both))),
-                    "sub" => Ok(BaselineShift(RsvgLength::new(-0.2, LengthUnit::Percent, LengthDir::Both))),
-                    "super" => Ok(BaselineShift(RsvgLength::new(0.4, LengthUnit::Percent, LengthDir::Both))),
-                    _ => Ok(BaselineShift(RsvgLength::parse(s, LengthDir::Both)?)),
+            fn parse(parser: &mut Parser, _: Self::Data) -> Result<BaselineShift, ::error::AttributeError> {
+                let parser_state = parser.state();
+
+                {
+                    let token = parser.next().map_err(|_| ::error::AttributeError::Parse(
+                        ::parsers::ParseError::new("expected token"),
+                    ))?;
+
+                    if let Token::Ident(ref cow) = token {
+                        match cow.as_ref() {
+                            "baseline" => return Ok(BaselineShift(
+                                RsvgLength::new(0.0, LengthUnit::Percent, LengthDir::Both)
+                            )),
+
+                            "sub" => return Ok(BaselineShift(
+                                RsvgLength::new(-0.2, LengthUnit::Percent, LengthDir::Both)
+                            )),
+
+                            "super" => return Ok(BaselineShift(
+                                RsvgLength::new(0.4, LengthUnit::Percent, LengthDir::Both),
+                            )),
+
+                            _ => (),
+                        }
+                    }
                 }
+
+                parser.reset(&parser_state);
+
+                Ok(BaselineShift(RsvgLength::from_cssparser(parser, LengthDir::Both)?))
             }
         }
     }
@@ -794,6 +780,19 @@ make_property!(
     parse_data_type: ()
 );
 
+// https://www.w3.org/TR/SVG11/painting.html#ColorInterpolationProperty
+make_property!(
+    ComputedValues,
+    ColorInterpolationFilters,
+    default: LinearRgb,
+    inherits_automatically: true,
+
+    identifiers:
+    "auto" => Auto,
+    "linearRGB" => LinearRgb,
+    "sRGB" => Srgb,
+);
+
 // https://gitlab.gnome.org/GNOME/librsvg/issues/268 - can we remove this property?
 make_property!(
     ComputedValues,
@@ -883,7 +882,7 @@ make_property!(
 make_property!(
     ComputedValues,
     Fill,
-    default: PaintServer::parse("#000", ()).unwrap(),
+    default: PaintServer::parse_str("#000", ()).unwrap(),
     inherits_automatically: true,
     newtype_parse: PaintServer,
     parse_data_type: ()
@@ -895,7 +894,8 @@ make_property!(
     FillOpacity,
     default: UnitInterval(1.0),
     inherits_automatically: true,
-    newtype_from_str: UnitInterval
+    newtype_parse: UnitInterval,
+    parse_data_type: ()
 );
 
 // https://www.w3.org/TR/SVG/painting.html#FillRuleProperty
@@ -936,7 +936,8 @@ make_property!(
     FloodOpacity,
     default: UnitInterval(1.0),
     inherits_automatically: false,
-    newtype_from_str: UnitInterval
+    newtype_parse: UnitInterval,
+    parse_data_type: ()
 );
 
 // https://www.w3.org/TR/SVG/text.html#FontFamilyProperty
@@ -945,14 +946,15 @@ make_property!(
     FontFamily,
     default: "Times New Roman".to_string(),
     inherits_automatically: true,
-    newtype_from_str: String
+    newtype_parse: String,
+    parse_data_type: ()
 );
 
 // https://www.w3.org/TR/SVG/text.html#FontSizeProperty
 make_property!(
     ComputedValues,
     FontSize,
-    default: RsvgLength::parse("12.0", LengthDir::Both).unwrap(),
+    default: RsvgLength::parse_str("12.0", LengthDir::Both).unwrap(),
     newtype_parse: RsvgLength,
     parse_data_type: LengthDir,
     property_impl: {
@@ -1114,7 +1116,8 @@ make_property!(
     Opacity,
     default: UnitInterval(1.0),
     inherits_automatically: false,
-    newtype_from_str: UnitInterval
+    newtype_parse: UnitInterval,
+    parse_data_type: ()
 );
 
 // https://www.w3.org/TR/SVG/masking.html#OverflowProperty
@@ -1161,7 +1164,8 @@ make_property!(
     StopOpacity,
     default: UnitInterval(1.0),
     inherits_automatically: false,
-    newtype_from_str: UnitInterval
+    newtype_parse: UnitInterval,
+    parse_data_type: ()
 );
 
 // https://www.w3.org/TR/SVG/painting.html#StrokeProperty
@@ -1226,7 +1230,8 @@ make_property!(
     StrokeMiterlimit,
     default: 4f64,
     inherits_automatically: true,
-    newtype_from_str: f64
+    newtype_parse: f64,
+    parse_data_type: ()
 );
 
 // https://www.w3.org/TR/SVG/painting.html#StrokeOpacityProperty
@@ -1235,14 +1240,15 @@ make_property!(
     StrokeOpacity,
     default: UnitInterval(1.0),
     inherits_automatically: true,
-    newtype_from_str: UnitInterval
+    newtype_parse: UnitInterval,
+    parse_data_type: ()
 );
 
 // https://www.w3.org/TR/SVG/painting.html#StrokeWidthProperty
 make_property!(
     ComputedValues,
     StrokeWidth,
-    default: RsvgLength::parse("1.0", LengthDir::Both).unwrap(),
+    default: RsvgLength::parse_str("1.0", LengthDir::Both).unwrap(),
     inherits_automatically: true,
     newtype_parse: RsvgLength,
     parse_data_type: LengthDir
@@ -1278,17 +1284,89 @@ make_property!(
             type Data = ();
             type Err = AttributeError;
 
-            fn parse(s: &str, _: Self::Data) -> Result<TextDecoration, AttributeError> {
+            fn parse(parser: &mut Parser, _: Self::Data) -> Result<TextDecoration, AttributeError> {
+                let mut overline = false;
+                let mut underline = false;
+                let mut strike = false;
+
+                if parser.try(|p| p.expect_ident_matching("none")).is_ok() {
+                    return Ok(TextDecoration::default());
+                }
+
+                while !parser.is_exhausted() {
+                    let cow = parser.expect_ident().map_err(|_| ::error::AttributeError::Parse(
+                        ::parsers::ParseError::new("expected identifier"),
+                    ))?;
+
+                    match cow.as_ref() {
+                        "overline" => overline = true,
+                        "underline" => underline = true,
+                        "line-through" => strike = true,
+                        _ => return Err(AttributeError::Parse(ParseError::new("invalid syntax"))),
+                    }
+                }
+
                 Ok(TextDecoration {
-                    overline: s.contains("overline"),
-                    underline: s.contains("underline"),
-                    strike: s.contains("strike") || s.contains("line-through"),
+                    overline,
+                    underline,
+                    strike,
                 })
             }
         }
     }
 );
 
+#[cfg(test)]
+#[test]
+fn parses_text_decoration() {
+    assert_eq!(
+        TextDecoration::parse_str("none", ()).unwrap(),
+        TextDecoration {
+            overline: false,
+            underline: false,
+            strike: false,
+        }
+    );
+
+    assert_eq!(
+        TextDecoration::parse_str("overline", ()).unwrap(),
+        TextDecoration {
+            overline: true,
+            underline: false,
+            strike: false,
+        }
+    );
+
+    assert_eq!(
+        TextDecoration::parse_str("underline", ()).unwrap(),
+        TextDecoration {
+            overline: false,
+            underline: true,
+            strike: false,
+        }
+    );
+
+    assert_eq!(
+        TextDecoration::parse_str("line-through", ()).unwrap(),
+        TextDecoration {
+            overline: false,
+            underline: false,
+            strike: true,
+        }
+    );
+
+    assert_eq!(
+        TextDecoration::parse_str("underline overline", ()).unwrap(),
+        TextDecoration {
+            overline: true,
+            underline: true,
+            strike: false,
+        }
+    );
+
+    assert!(TextDecoration::parse_str("airline", ()).is_err())
+}
+
 // https://www.w3.org/TR/SVG/painting.html#TextRenderingProperty
 make_property!(
     ComputedValues,
@@ -1350,7 +1428,8 @@ make_property!(
     XmlLang,
     default: "".to_string(), // see create_pango_layout()
     inherits_automatically: true,
-    newtype_from_str: String
+    newtype_parse: String,
+    parse_data_type: ()
 );
 
 make_property!(
@@ -1471,14 +1550,6 @@ pub fn parse_style_attrs(
          *        } */
     }
 
-    match state.parse_conditional_processing_attributes(pbag) {
-        Ok(_) => (),
-        Err(e) => {
-            node.set_error(e);
-            return;
-        }
-    }
-
     // Try to properly support all of the following, including inheritance:
     // *
     // #id
@@ -1570,25 +1641,6 @@ pub fn parse_style_attrs(
     }
 }
 
-#[no_mangle]
-pub extern "C" fn rsvg_computed_values_get_flood_color_argb(values: RsvgComputedValues) -> u32 {
-    assert!(!values.is_null());
-    let values = unsafe { &*values };
-
-    match values.flood_color {
-        FloodColor(cssparser::Color::CurrentColor) => rgba_to_argb(values.color.0),
-        FloodColor(cssparser::Color::RGBA(ref rgba)) => rgba_to_argb(*rgba),
-    }
-}
-
-#[no_mangle]
-pub extern "C" fn rsvg_computed_values_get_flood_opacity(values: RsvgComputedValues) -> u8 {
-    assert!(!values.is_null());
-    let values = unsafe { &*values };
-
-    u8::from(values.flood_opacity.0)
-}
-
 #[no_mangle]
 pub extern "C" fn rsvg_computed_values_get_lighting_color_argb(values: RsvgComputedValues) -> u32 {
     assert!(!values.is_null());
