@@ -3,13 +3,12 @@ use std::cell::RefCell;
 
 use aspect_ratio::*;
 use attributes::Attribute;
-use drawing_ctx;
-use drawing_ctx::RsvgDrawingCtx;
+use drawing_ctx::DrawingCtx;
 use float_eq_cairo::ApproxEqCairo;
 use handle::RsvgHandle;
 use length::*;
 use node::*;
-use parsers::{parse, Parse};
+use parsers::{parse, parse_and_validate, Parse};
 use property_bag::{OwnedPropertyBag, PropertyBag};
 use state::{self, Overflow};
 use viewbox::*;
@@ -32,11 +31,14 @@ impl NodeTrait for NodeGroup {
         &self,
         node: &RsvgNode,
         cascaded: &CascadedValues,
-        draw_ctx: *mut RsvgDrawingCtx,
-        with_layer: bool,
+        draw_ctx: &mut DrawingCtx,
         clipping: bool,
     ) {
-        node.draw_children(node, cascaded, draw_ctx, with_layer, clipping);
+        let values = cascaded.get();
+
+        draw_ctx.with_discrete_layer(node, values, clipping, &mut |dc| {
+            node.draw_children(cascaded, dc, clipping);
+        });
     }
 }
 
@@ -71,25 +73,16 @@ impl NodeTrait for NodeSwitch {
         &self,
         node: &RsvgNode,
         cascaded: &CascadedValues,
-        draw_ctx: *mut RsvgDrawingCtx,
-        _with_layer: bool,
+        draw_ctx: &mut DrawingCtx,
         clipping: bool,
     ) {
         let values = cascaded.get();
 
-        drawing_ctx::push_discrete_layer(draw_ctx, values, clipping);
-
-        if let Some(child) = node.children().find(|c| c.get_state().cond) {
-            drawing_ctx::draw_node_from_stack(
-                draw_ctx,
-                &CascadedValues::new(cascaded, &child),
-                &child,
-                true,
-                clipping,
-            );
-        }
-
-        drawing_ctx::pop_discrete_layer(draw_ctx, node, values, clipping);
+        draw_ctx.with_discrete_layer(node, values, clipping, &mut |dc| {
+            if let Some(child) = node.children().find(|c| c.get_cond()) {
+                dc.draw_node_from_stack(&CascadedValues::new(cascaded, &child), &child, clipping);
+            }
+        });
     }
 }
 
@@ -107,10 +100,10 @@ impl NodeSvg {
     pub fn new() -> NodeSvg {
         NodeSvg {
             preserve_aspect_ratio: Cell::new(AspectRatio::default()),
-            x: Cell::new(RsvgLength::parse("0", LengthDir::Horizontal).unwrap()),
-            y: Cell::new(RsvgLength::parse("0", LengthDir::Vertical).unwrap()),
-            w: Cell::new(RsvgLength::parse("100%", LengthDir::Horizontal).unwrap()),
-            h: Cell::new(RsvgLength::parse("100%", LengthDir::Vertical).unwrap()),
+            x: Cell::new(RsvgLength::parse_str("0", LengthDir::Horizontal).unwrap()),
+            y: Cell::new(RsvgLength::parse_str("0", LengthDir::Vertical).unwrap()),
+            w: Cell::new(RsvgLength::parse_str("100%", LengthDir::Horizontal).unwrap()),
+            h: Cell::new(RsvgLength::parse_str("100%", LengthDir::Vertical).unwrap()),
             vbox: Cell::new(None),
             pbag: RefCell::new(None),
         }
@@ -131,36 +124,36 @@ impl NodeTrait for NodeSvg {
             match attr {
                 Attribute::PreserveAspectRatio => {
                     self.preserve_aspect_ratio
-                        .set(parse("preserveAspectRatio", value, (), None)?)
+                        .set(parse("preserveAspectRatio", value, ())?)
                 }
 
                 Attribute::X => {
                     if is_inner_svg {
-                        self.x.set(parse("x", value, LengthDir::Horizontal, None)?);
+                        self.x.set(parse("x", value, LengthDir::Horizontal)?);
                     }
                 }
 
                 Attribute::Y => {
                     if is_inner_svg {
-                        self.y.set(parse("y", value, LengthDir::Vertical, None)?);
+                        self.y.set(parse("y", value, LengthDir::Vertical)?);
                     }
                 }
 
-                Attribute::Width => self.w.set(parse(
+                Attribute::Width => self.w.set(parse_and_validate(
                     "width",
                     value,
                     LengthDir::Horizontal,
-                    Some(RsvgLength::check_nonnegative),
+                    RsvgLength::check_nonnegative,
                 )?),
 
-                Attribute::Height => self.h.set(parse(
+                Attribute::Height => self.h.set(parse_and_validate(
                     "height",
                     value,
                     LengthDir::Vertical,
-                    Some(RsvgLength::check_nonnegative),
+                    RsvgLength::check_nonnegative,
                 )?),
 
-                Attribute::ViewBox => self.vbox.set(parse("viewBox", value, (), None).map(Some)?),
+                Attribute::ViewBox => self.vbox.set(parse("viewBox", value, ()).map(Some)?),
 
                 _ => (),
             }
@@ -177,8 +170,7 @@ impl NodeTrait for NodeSvg {
         &self,
         node: &RsvgNode,
         cascaded: &CascadedValues,
-        draw_ctx: *mut RsvgDrawingCtx,
-        _with_layer: bool,
+        draw_ctx: &mut DrawingCtx,
         clipping: bool,
     ) {
         let values = cascaded.get();
@@ -201,11 +193,12 @@ impl NodeTrait for NodeSvg {
             self.preserve_aspect_ratio.get(),
             node,
             values,
-            drawing_ctx::get_cairo_context(draw_ctx).get_matrix(),
+            draw_ctx.get_cairo_context().get_matrix(),
             draw_ctx,
             clipping,
-            || {
-                node.draw_children(node, cascaded, draw_ctx, false, clipping);
+            &mut |dc| {
+                // we don't push a layer because draw_in_viewport() already does it
+                node.draw_children(cascaded, dc, clipping);
             },
         );
     }
@@ -237,23 +230,23 @@ impl NodeTrait for NodeUse {
             match attr {
                 Attribute::XlinkHref => *self.link.borrow_mut() = Some(value.to_owned()),
 
-                Attribute::X => self.x.set(parse("x", value, LengthDir::Horizontal, None)?),
-                Attribute::Y => self.y.set(parse("y", value, LengthDir::Vertical, None)?),
+                Attribute::X => self.x.set(parse("x", value, LengthDir::Horizontal)?),
+                Attribute::Y => self.y.set(parse("y", value, LengthDir::Vertical)?),
 
                 Attribute::Width => self.w.set(
-                    parse(
+                    parse_and_validate(
                         "width",
                         value,
                         LengthDir::Horizontal,
-                        Some(RsvgLength::check_nonnegative),
+                        RsvgLength::check_nonnegative,
                     ).map(Some)?,
                 ),
                 Attribute::Height => self.h.set(
-                    parse(
+                    parse_and_validate(
                         "height",
                         value,
                         LengthDir::Vertical,
-                        Some(RsvgLength::check_nonnegative),
+                        RsvgLength::check_nonnegative,
                     ).map(Some)?,
                 ),
 
@@ -268,8 +261,7 @@ impl NodeTrait for NodeUse {
         &self,
         node: &RsvgNode,
         cascaded: &CascadedValues,
-        draw_ctx: *mut RsvgDrawingCtx,
-        _with_layer: bool,
+        draw_ctx: &mut DrawingCtx,
         clipping: bool,
     ) {
         let values = cascaded.get();
@@ -280,9 +272,7 @@ impl NodeTrait for NodeUse {
             return;
         }
 
-        let child = if let Some(acquired) =
-            drawing_ctx::get_acquired_node(draw_ctx, link.as_ref().unwrap())
-        {
+        let child = if let Some(acquired) = draw_ctx.get_acquired_node(link.as_ref().unwrap()) {
             acquired.get()
         } else {
             return;
@@ -304,12 +294,12 @@ impl NodeTrait for NodeUse {
         let nw = self
             .w
             .get()
-            .unwrap_or_else(|| RsvgLength::parse("100%", LengthDir::Horizontal).unwrap())
+            .unwrap_or_else(|| RsvgLength::parse_str("100%", LengthDir::Horizontal).unwrap())
             .normalize(values, draw_ctx);
         let nh = self
             .h
             .get()
-            .unwrap_or_else(|| RsvgLength::parse("100%", LengthDir::Vertical).unwrap())
+            .unwrap_or_else(|| RsvgLength::parse_str("100%", LengthDir::Vertical).unwrap())
             .normalize(values, draw_ctx);
 
         // width or height set to 0 disables rendering of the element
@@ -319,24 +309,21 @@ impl NodeTrait for NodeUse {
         }
 
         if child.get_type() != NodeType::Symbol {
-            let cr = drawing_ctx::get_cairo_context(draw_ctx);
+            let cr = draw_ctx.get_cairo_context();
             cr.translate(nx, ny);
 
-            drawing_ctx::push_discrete_layer(draw_ctx, values, clipping);
-
-            drawing_ctx::draw_node_from_stack(
-                draw_ctx,
-                &CascadedValues::new_from_values(&child, values),
-                &child,
-                true,
-                clipping,
-            );
-
-            drawing_ctx::pop_discrete_layer(draw_ctx, node, values, clipping);
+            draw_ctx.with_discrete_layer(node, values, clipping, &mut |dc| {
+                dc.draw_node_from_stack(
+                    &CascadedValues::new_from_values(&child, values),
+                    &child,
+                    clipping,
+                );
+            });
         } else {
             child.with_impl(|symbol: &NodeSymbol| {
                 let do_clip = !values.is_overflow()
-                    || (values.overflow == Overflow::Visible && child.get_state().is_overflow());
+                    || (values.overflow == Overflow::Visible
+                        && child.get_specified_values().is_overflow());
 
                 draw_in_viewport(
                     nx,
@@ -349,19 +336,16 @@ impl NodeTrait for NodeUse {
                     symbol.preserve_aspect_ratio.get(),
                     node,
                     values,
-                    drawing_ctx::get_cairo_context(draw_ctx).get_matrix(),
+                    draw_ctx.get_cairo_context().get_matrix(),
                     draw_ctx,
                     clipping,
-                    || {
-                        drawing_ctx::push_discrete_layer(draw_ctx, values, clipping);
+                    &mut |dc| {
+                        // We don't push a layer because draw_in_viewport() already does it
                         child.draw_children(
-                            &child,
                             &CascadedValues::new_from_values(&child, values),
-                            draw_ctx,
-                            false,
+                            dc,
                             clipping,
                         );
-                        drawing_ctx::pop_discrete_layer(draw_ctx, node, values, clipping);
                     },
                 );
             });
@@ -393,10 +377,10 @@ impl NodeTrait for NodeSymbol {
             match attr {
                 Attribute::PreserveAspectRatio => {
                     self.preserve_aspect_ratio
-                        .set(parse("preserveAspectRatio", value, (), None)?)
+                        .set(parse("preserveAspectRatio", value, ())?)
                 }
 
-                Attribute::ViewBox => self.vbox.set(parse("viewBox", value, (), None).map(Some)?),
+                Attribute::ViewBox => self.vbox.set(parse("viewBox", value, ()).map(Some)?),
 
                 _ => (),
             }
