@@ -6,7 +6,7 @@ use std::str;
 
 use attributes::Attribute;
 use draw::draw_pango_layout;
-use drawing_ctx::{self, RsvgDrawingCtx};
+use drawing_ctx::DrawingCtx;
 use handle::RsvgHandle;
 use length::*;
 use node::{boxed_node_new, CascadedValues, NodeResult, NodeTrait, NodeType, RsvgNode};
@@ -67,7 +67,7 @@ impl NodeChars {
         &self,
         _node: &RsvgNode,
         values: &ComputedValues,
-        draw_ctx: *const RsvgDrawingCtx,
+        draw_ctx: &DrawingCtx,
         length: &mut f64,
     ) {
         let s = self.string.borrow();
@@ -81,7 +81,7 @@ impl NodeChars {
         &self,
         _node: &RsvgNode,
         values: &ComputedValues,
-        draw_ctx: *mut RsvgDrawingCtx,
+        draw_ctx: &mut DrawingCtx,
         x: &mut f64,
         y: &mut f64,
         clipping: bool,
@@ -131,12 +131,10 @@ impl NodeTrait for NodeText {
     fn set_atts(&self, _: &RsvgNode, _: *const RsvgHandle, pbag: &PropertyBag) -> NodeResult {
         for (_key, attr, value) in pbag.iter() {
             match attr {
-                Attribute::X => self.x.set(parse("x", value, LengthDir::Horizontal, None)?),
-                Attribute::Y => self.y.set(parse("y", value, LengthDir::Vertical, None)?),
-                Attribute::Dx => self
-                    .dx
-                    .set(parse("dx", value, LengthDir::Horizontal, None)?),
-                Attribute::Dy => self.dy.set(parse("dy", value, LengthDir::Vertical, None)?),
+                Attribute::X => self.x.set(parse("x", value, LengthDir::Horizontal)?),
+                Attribute::Y => self.y.set(parse("y", value, LengthDir::Vertical)?),
+                Attribute::Dx => self.dx.set(parse("dx", value, LengthDir::Horizontal)?),
+                Attribute::Dy => self.dy.set(parse("dy", value, LengthDir::Vertical)?),
                 _ => (),
             }
         }
@@ -148,8 +146,7 @@ impl NodeTrait for NodeText {
         &self,
         node: &RsvgNode,
         cascaded: &CascadedValues,
-        draw_ctx: *mut RsvgDrawingCtx,
-        _with_layer: bool,
+        draw_ctx: &mut DrawingCtx,
         clipping: bool,
     ) {
         let values = cascaded.get();
@@ -201,7 +198,7 @@ impl NodeTRef {
         &self,
         _node: &RsvgNode,
         cascaded: &CascadedValues,
-        draw_ctx: *mut RsvgDrawingCtx,
+        draw_ctx: &mut DrawingCtx,
         length: &mut f64,
     ) -> bool {
         let l = self.link.borrow();
@@ -210,13 +207,12 @@ impl NodeTRef {
             return false;
         }
 
-        let done =
-            if let Some(acquired) = drawing_ctx::get_acquired_node(draw_ctx, l.as_ref().unwrap()) {
-                let c = acquired.get();
-                measure_children(&c, cascaded, draw_ctx, length, true)
-            } else {
-                false
-            };
+        let done = if let Some(acquired) = draw_ctx.get_acquired_node(l.as_ref().unwrap()) {
+            let c = acquired.get();
+            measure_children(&c, cascaded, draw_ctx, length, true)
+        } else {
+            false
+        };
 
         done
     }
@@ -225,7 +221,7 @@ impl NodeTRef {
         &self,
         _node: &RsvgNode,
         cascaded: &CascadedValues,
-        draw_ctx: *mut RsvgDrawingCtx,
+        draw_ctx: &mut DrawingCtx,
         x: &mut f64,
         y: &mut f64,
         clipping: bool,
@@ -236,7 +232,7 @@ impl NodeTRef {
             return;
         }
 
-        if let Some(acquired) = drawing_ctx::get_acquired_node(draw_ctx, l.as_ref().unwrap()) {
+        if let Some(acquired) = draw_ctx.get_acquired_node(l.as_ref().unwrap()) {
             let c = acquired.get();
             render_children(&c, cascaded, draw_ctx, x, y, true, clipping)
         }
@@ -277,7 +273,7 @@ impl NodeTSpan {
         &self,
         node: &RsvgNode,
         cascaded: &CascadedValues,
-        draw_ctx: *mut RsvgDrawingCtx,
+        draw_ctx: &mut DrawingCtx,
         length: &mut f64,
         usetextonly: bool,
     ) -> bool {
@@ -300,7 +296,7 @@ impl NodeTSpan {
         &self,
         node: &RsvgNode,
         cascaded: &CascadedValues,
-        draw_ctx: *mut RsvgDrawingCtx,
+        draw_ctx: &mut DrawingCtx,
         x: &mut f64,
         y: &mut f64,
         usetextonly: bool,
@@ -352,14 +348,12 @@ impl NodeTrait for NodeTSpan {
             match attr {
                 Attribute::X => self
                     .x
-                    .set(parse("x", value, LengthDir::Horizontal, None).map(Some)?),
+                    .set(parse("x", value, LengthDir::Horizontal).map(Some)?),
                 Attribute::Y => self
                     .y
-                    .set(parse("y", value, LengthDir::Vertical, None).map(Some)?),
-                Attribute::Dx => self
-                    .dx
-                    .set(parse("dx", value, LengthDir::Horizontal, None)?),
-                Attribute::Dy => self.dy.set(parse("dy", value, LengthDir::Vertical, None)?),
+                    .set(parse("y", value, LengthDir::Vertical).map(Some)?),
+                Attribute::Dx => self.dx.set(parse("dx", value, LengthDir::Horizontal)?),
+                Attribute::Dy => self.dy.set(parse("dy", value, LengthDir::Vertical)?),
                 _ => (),
             }
         }
@@ -470,11 +464,11 @@ impl From<WritingMode> for pango::Gravity {
 }
 
 fn create_pango_layout(
-    draw_ctx: *const RsvgDrawingCtx,
+    draw_ctx: &DrawingCtx,
     values: &ComputedValues,
     text: &str,
 ) -> pango::Layout {
-    let pango_context = drawing_ctx::get_pango_context(draw_ctx);
+    let pango_context = draw_ctx.get_pango_context();
 
     // See the construction of the XmlLang property
     // We use "" there as the default value; this means that the language is not set.
@@ -512,7 +506,7 @@ fn create_pango_layout(
 
     font_desc.set_stretch(pango::Stretch::from(values.font_stretch));
 
-    let (_, dpi_y) = drawing_ctx::get_dpi(draw_ctx);
+    let (_, dpi_y) = draw_ctx.get_dpi();
     font_desc.set_size(to_pango_units(
         values.font_size.0.normalize(values, draw_ctx) / dpi_y * 72.0,
     ));
@@ -549,7 +543,7 @@ fn create_pango_layout(
 fn anchor_offset(
     node: &RsvgNode,
     cascaded: &CascadedValues,
-    draw_ctx: *mut RsvgDrawingCtx,
+    draw_ctx: &mut DrawingCtx,
     anchor: TextAnchor,
     textonly: bool,
 ) -> f64 {
@@ -572,7 +566,7 @@ fn anchor_offset(
 fn measure_children(
     node: &RsvgNode,
     cascaded: &CascadedValues,
-    draw_ctx: *mut RsvgDrawingCtx,
+    draw_ctx: &mut DrawingCtx,
     length: &mut f64,
     textonly: bool,
 ) -> bool {
@@ -597,7 +591,7 @@ fn measure_children(
 fn measure_child(
     node: &RsvgNode,
     cascaded: &CascadedValues,
-    draw_ctx: *mut RsvgDrawingCtx,
+    draw_ctx: &mut DrawingCtx,
     length: &mut f64,
     textonly: bool,
 ) -> bool {
@@ -605,7 +599,7 @@ fn measure_child(
 
     let mut done = false;
 
-    let cr = drawing_ctx::get_cairo_context(draw_ctx);
+    let cr = draw_ctx.get_cairo_context();
     cr.save();
 
     cr.transform(node.get_transform());
@@ -654,7 +648,7 @@ fn measure_child(
 fn render_children(
     node: &RsvgNode,
     cascaded: &CascadedValues,
-    draw_ctx: *mut RsvgDrawingCtx,
+    draw_ctx: &mut DrawingCtx,
     x: &mut f64,
     y: &mut f64,
     textonly: bool,
@@ -662,19 +656,17 @@ fn render_children(
 ) {
     let values = cascaded.get();
 
-    drawing_ctx::push_discrete_layer(draw_ctx, values, clipping);
-
-    for child in node.children() {
-        render_child(&child, cascaded, draw_ctx, x, y, textonly, clipping);
-    }
-
-    drawing_ctx::pop_discrete_layer(draw_ctx, node, values, clipping);
+    draw_ctx.with_discrete_layer(node, values, clipping, &mut |dc| {
+        for child in node.children() {
+            render_child(&child, cascaded, dc, x, y, textonly, clipping);
+        }
+    });
 }
 
 fn render_child(
     node: &RsvgNode,
     cascaded: &CascadedValues,
-    draw_ctx: *mut RsvgDrawingCtx,
+    draw_ctx: &mut DrawingCtx,
     x: &mut f64,
     y: &mut f64,
     textonly: bool,
@@ -682,7 +674,7 @@ fn render_child(
 ) {
     let values = cascaded.get();
 
-    let cr = drawing_ctx::get_cairo_context(draw_ctx);
+    let cr = draw_ctx.get_cairo_context();
     cr.save();
 
     cr.transform(node.get_transform());
